#------------------------------------------------------------------------
ThisCommandFile = 'Setup_Readout_Crate.cmd'
#
#  Used to setup the Run IIb L1Cal Trig readout via VME I/O
#
# Created  20-MAY-05  From the Run IIa L1Cal Init_Post_Auxi_L1CT.vio
# Modified  8-DEC-05  Investigate apparent setup problem (by comparing to excel setup)
#                     - write 0xff0b instead of 0xff0a 
#                       into the SBC register for the (unused) "high word of address"
#                     - setup the VRBs not being readout to also expect no input
#                       (they are also already setup to not participate)
#                     - explicitely clear the gray code control register of all VRBs 
#                     Still different; what this file *does* and excel setup *does not*:
#                     - reset and setup the VRBC for PDAQ
#                     - setup each VRB to enable/disable readout
#
# Modified  02-Feb-2006  Add support for VRB emulation mode
#
#------------------------------------------------------------------------

# -------------------------------------------------------------------------------
# ---  This section directs the behavior of the rest of the command file
# ---  The operator needs to specify 
# ---       the location and number of VRBs included in the readout, and in the crate
# ---       the set of VTM channels included in the readout for each VRB
# ---       whether to use the emulation mode supported by the VRB
# -------------------------------------------------------------------------------


# Specify the First Slot populated with a VRB.  
# Additional VRBs are expected to be in the following adjacent slot(s) 

VrbSlotMin    = 9 



# Specify the Number of VRBs which will be told to readout

VrbReadoutTot = 3  



# Specify the Grand Total number of VRBs present in the crate; 
# unused VRBs will be told not to readout
# note: VrbInCrateTot must thus be greater or equal to VrbReadoutTot 

VrbInCrateTot = 3  



# Specify which VRB/VTC channels are actually being readout.
# The variable EnabledVrbChan is a two dimensional array, with dimensions [SlotNum][ChannelPair]
# There must be (at least) n=VrbReadoutTot values defined for the first dimension (SlotNum).
# There must be exactly 4 Channel Pairs (0/1, 2/3, 4/5, 6/7) defined per VRB for the second dimension
#
# 0 = disable this channel pair (i.e. will not participate in the readout)
# 1 =  enable this channel pair (i.e. will participate in the readout)
# example:
# EnabledVrbChan=[ [ 1,1,1,1 ],  # All Channels Emabled for the 4 channel pairs of VRB in Slot# VrbSlotMin
#                  [ 1,0,0,0 ],  # Only Channel Pair 0/1 Enabled for the VRB in Slot#VrbSlotMin+1
#                  [ 0,0,0,1 ] ] # Only Channel Pair 6/7 Enabled for the VRB in Slot# VrbSlotMin+2

EnabledVrbChan = [ [ 1,1,1,0 ],
                   [ 1,1,1,0 ],
                   [ 1,1,1,0 ] ] 


# Specify the VRB mode of operation
# 
# 0 = normal readout mode (i.e. use the data from the TABs)
# 1 = emulation mode (i.e. use the VRB's emulation data of 256 bytes 0:255 on each channel)
#
# Note: the VRB hardware allows for individually setting each channel in emulation vs normal mode
#       but this command file configures all enabled channels to always operate in a common mode 

SetVrbEmulationMode = 0



# -------------------------------------------------------------------------------
# ---  The rest of the command file does not need to be modified
# ---  All the control were specified above
# -------------------------------------------------------------------------------


# Vertical IO VME Addresses

A32VIMaster1Slave0BaseAddress = 0x14000000  # Base address for master #1 slave #0 
A24OffsetForShortIOSpace      =   0xFF0000  # the upper 64k addresses generate A16 cycles in VI slave crate


# -------------------------------------------------------------------------------
#
# VBD Setup 
# ---------
#
# Note: The VBD functionality is now provided by the crate's SBC
#       Some of the actions below intended for VBD control 
#       may not be strictly required, but must be by definition 
#       compatible with the SBC setup.
#
# The VBD will be configured for reading out N VRBs located in consecutive slots
# Its base address is 0x6000 and the registers are accesible in A16/D16 VME cycles.

# The VME base address of the VBD/SBC in short IO space is 0x6000

A16VbdConfigBaseAddress = 0x6000  

VbdConfigBaseVmeAddr  = A32VIMaster1Slave0BaseAddress \
                      + A24OffsetForShortIOSpace      \
                      + A16VbdConfigBaseAddress

# VBD register addresses

Vbd_CSR0_VmeAddr         = VbdConfigBaseVmeAddr + 0x00
Vbd_CrateType_VmeAddr    = VbdConfigBaseVmeAddr + 0x08
Vbd_EventNumber_VmeAddr  = VbdConfigBaseVmeAddr + 0x0a
Vbd_CrateID_VmeAddr      = VbdConfigBaseVmeAddr + 0x0c
Vbd_DControl_VmeAddr     = VbdConfigBaseVmeAddr + 0x10
Vbd_PControl_VmeAddr     = VbdConfigBaseVmeAddr + 0x12
Vbd_IOAddress_VmeAddr    = VbdConfigBaseVmeAddr + 0x14
Vbd_WCPointer_VmeAddr    = VbdConfigBaseVmeAddr + 0x1000
Vbd_DataPointer_VmeAddr  = VbdConfigBaseVmeAddr + 0x1800


# Enable writing into control memory (i.e. unlock the board)

Vme_Write ( VmeAddress = Vbd_CSR0_VmeAddr, 
               DataOut = 0x000C )   


# Set Central Tracking mode

Vme_Write ( VmeAddress = Vbd_CrateType_VmeAddr, 
               DataOut = 0x0002 )   


# Address where the event number can be found (in short I/O space on the VBRC)

Vme_Write ( VmeAddress = Vbd_EventNumber_VmeAddr, 
               DataOut = 0xC0D2 )   


# Address where the Crate ID can be found (in short I/O space on the VBRC)

Vme_Write ( VmeAddress = Vbd_CrateID_VmeAddr, 
               DataOut = 0xC0D0 )   


# Code for add. modifier for D Control (i.e. for reading data)

Vme_Write ( VmeAddress = Vbd_DControl_VmeAddr, 
               DataOut = 0xBBF9 )   


# Code for Address modifier for P Control (i.e. for Crate ID/Event number access)

Vme_Write ( VmeAddress = Vbd_PControl_VmeAddr, 
               DataOut = 0xEDED )   


# High Word to use for addresses
# In fact not used, since the P control was configured in A16/D16 memory space

Vme_Write ( VmeAddress = Vbd_IOAddress_VmeAddr, 
               DataOut = 0xFF0B )   



# Setup the Word Count list 

for VrbNum in range(VrbReadoutTot) :

    # Add Nth VRB: The word count is at (base address + this offset)
    Vme_Write ( VmeAddress = Vbd_WCPointer_VmeAddr + VrbNum * 0x02, 
                   DataOut = 0x0032 )   

# Null terminate the Word Count list after the last VRB  
Vme_Write ( VmeAddress = Vbd_WCPointer_VmeAddr + VrbReadoutTot * 0x02, 
               DataOut = 0x0000 )   



# Setup the Data Address List

for VrbNum in range(VrbReadoutTot) :

    # Pointer to data (High 16 bits of Nth VRB) 
    Vme_Write ( VmeAddress = Vbd_DataPointer_VmeAddr + VrbNum * 0x04, 
                   DataOut = VrbSlotMin + VrbNum )   

    # Pointer to data (Low 16 bits of Nth VRB) 
    Vme_Write ( VmeAddress = Vbd_DataPointer_VmeAddr + VrbNum * 0x04 + 0x02, 
                   DataOut = 0x0018 )   



# Lock memory and Reset the board

Vme_Write ( VmeAddress = Vbd_CSR0_VmeAddr, 
               DataOut = 0x0080 )   

# Lock the board 

Vme_Write ( VmeAddress = Vbd_CSR0_VmeAddr, 
               DataOut = 0x001C )   


# -------------------------------------------------------------------------------
#
# VRBC Setup
# ----------
#
# Four (4) registers on the VRB controller are relevant 
# for setting up the correct mode of operation of this particular
# Readout System. The sequence in which these commands are sent
# matters.

# The VME  base address for the VRB Controller in A24 space is 0x480000

A24VrbcConfigBaseAddress  =   0x480000

VrbcConfigBaseVmeAddr     = A32VIMaster1Slave0BaseAddress \
                          + A24VrbcConfigBaseAddress

# VRBC register addresses

Vrbc_Reset_VmeAddr        = VrbcConfigBaseVmeAddr + 0x0A
Vrbc_PathControl_VmeAddr  = VrbcConfigBaseVmeAddr + 0xB2
Vrbc_SlaveReady_VmeAddr   = VrbcConfigBaseVmeAddr + 0xD0


# This instruction performs a VRBC reset
# RESET: (VME address = 0x48000A)

Vme_Write ( VmeAddress = Vrbc_Reset_VmeAddr, 
               DataOut = 0x0000 )   

# wait for VRBC to get organized
time.sleep (1)


# Allows the VRBC to choose the path where event data will be moved to. Either 
# the Level 3 system (PDAQ) or to a embedded processor (SDAQ) residing in the 
# VIPA crate (if any). In our case only PDAQ will be used.
#
# PATH CONTROL: (VME Address = 0x4800B2)
#
# The bit assignment is as follows:
#
# Bits (1,0) -- (0,0) -- No path chosen
# Bits (1,0) -- (0,1) -- PDAQ
# Bits (1,0) -- (1,0) -- SDAQ
# Bita (1,0) -- (1,1) -- PDAQ and SDAQ
#
# Select PDAQ

Vme_Write ( VmeAddress = Vrbc_PathControl_VmeAddr, 
               DataOut = 0x0001 )   


# When working in SDAQ Mode, this register allows the VRBC to emulate the handshaking
# between the VBD and the VRBC since no data will be transferred to the VBD. This
# way the VBD won't be confused.
#
# SLAVE READY AND FINISH; (VME address = 0x4800D0)
#
# The bit assignment is as follows:
#
# Bit 0 - Slave Ready emulated - Active LOW
# Bit 1 - Finish emulated - Active LOW
#
# Both, Slave Ready and Finished are INACTIVE for PDAQ

Vme_Write ( VmeAddress = Vrbc_SlaveReady_VmeAddr, 
               DataOut = 0x0003 )   



# -------------------------------------------------------------------------------
#
# VRBs Setup
# ----------
#

# The base address of the VRB in A24 space is (SlotNum x this_address)

A24VrbModuloAddress       =   0x010000  

# Channel Enable
# --------------
#
# First. lets begin with enabling the  channels to be used
# For this particular test, only one channel of the 1st VRB (0x09XXXX) will 
# be enabled. The offset for this register is 0X0070
#
# The bit-assignmets for this register is as follows
#
# bit 0 - Channel enable 0 - active HIGH
# bit 1 - Channel enable 1 - active HIGH
# bit 2 - Channel enable 2 - active HIGH
# bit 3 - Channel enable 3 - active HIGH
# bit 4 - Channel enable 4 - active HIGH
# bit 5 - Channel enable 5 - active HIGH
# bit 6 - Channel enable 6 - active HIGH
# bit 7 - Channel enable 7 - active HIGH
# bits 8- 15 not used
#

# VRB Channel Enable address offset

Vrb_ChanEnable_AddrOffset = 0x70

# Channel Enable for the VRBs is driven by the array EnabledVrbChan at the top of this file

for VrbNum in range(VrbInCrateTot) :

    Vrb_ChanEnable_VmeAddr = A32VIMaster1Slave0BaseAddress \
                           + ( VrbSlotMin + VrbNum ) * A24VrbModuloAddress \
                           + Vrb_ChanEnable_AddrOffset

    Vrb_ChanEnable_Value = 0x00  # default: no channel being read out
    
    if ( VrbNum < VrbReadoutTot ) :
        for ChannelPair in range(4) :
            Vrb_ChanEnable_Value = Vrb_ChanEnable_Value \
                                    + 0x03 * 2**(2*ChannelPair) \
                                           * EnabledVrbChan[VrbNum][ChannelPair] 

    Vme_Write ( VmeAddress = Vrb_ChanEnable_VmeAddr, 
                   DataOut = Vrb_ChanEnable_Value )   
    


# Emulation Mode
# --------------
#
# Next, lets define if any of the enabled channels will be working in
# emulation mode
#
# The offset for this register is 0x0X0072
# The bit-assignmets for this register is as follows
#
# bit 0 - Channel 0  in emulation mode- active HIGH
# bit 1 - Channel 1      "            - active HIGH
# bit 2 - Channel 2      "            - active HIGH 
# bit 3 - Channel 3      "            - active HIGH
# bit 4 - Channel 4      "            - active HIGH
# bit 5 - Channel 5      "            - active HIGH
# bit 6 - Channel 6      "            - active HIGH
# bit 7 - Channel 7      "            - active HIGH
# bits 8- 15 not used
#

Vrb_ChanEmul_AddrOffset = 0x72

# emulation mode is disabled  for all the channels - VRB in slot 9

for VrbNum in range(VrbInCrateTot) :

    Vrb_ChanEmul_VmeAddr = A32VIMaster1Slave0BaseAddress \
                          + ( VrbSlotMin + VrbNum ) * A24VrbModuloAddress \
                          + Vrb_ChanEmul_AddrOffset

    Vrb_ChanEmul_Value = 0x00  # default: no channel in emulation mode

    if ( SetVrbEmulationMode == 1 ) :
        if ( VrbNum < VrbReadoutTot ) :
            for ChannelPair in range(4) :
                Vrb_ChanEmul_Value = Vrb_ChanEmul_Value \
                                        + 0x03 * 2**(2*ChannelPair) \
                                               * EnabledVrbChan[VrbNum][ChannelPair] 

    Vme_Write ( VmeAddress = Vrb_ChanEmul_VmeAddr, 
                   DataOut = Vrb_ChanEmul_Value )   
    


# Gray Code (not) Enable
# -----------------------
#

# VRB Gray Code Enable address offset

Vrb_GrayEnable_AddrOffset = 0x74

for VrbNum in range(VrbInCrateTot) :

    Vrb_GrayEnable_VmeAddr = A32VIMaster1Slave0BaseAddress \
                           + ( VrbSlotMin + VrbNum ) * A24VrbModuloAddress \
                           + Vrb_GrayEnable_AddrOffset

    Vrb_GrayEnable_Value = 0x00  # default: no channel using Gray Code
    
    Vme_Write ( VmeAddress = Vrb_GrayEnable_VmeAddr, 
                   DataOut = Vrb_GrayEnable_Value )   
    


# VRB Control
# -----------
#
# We need to tell each VRB if theay are going to be part of the readout. Otherwise
# they need to be disconnected from the J3 backplane so that any control message from the
# VRBC  will be ignored. The status signals from the J3 backplane should be disconnected
# for those VRBs not in use
#
# The offset for this register is 0x0X0040
# The bit assignmen of this register is as follows:
# bit 0 - Control port enable - HIGH if the VRB will respond to VRBC commands -- the default is HIGH
# bit 1 - Status Signal enable - HIGH if the VRB sends status signals to the VRB-- the default is HIGH
#

# VRB Control Register address offset

Vrb_Control_AddrOffset = 0x40

# Tell the first VrbReadoutTot VRBs to readout, and any VRB above that not to readout 

for VrbNum in range(VrbInCrateTot) :

    Vrb_Control_VmeAddr = A32VIMaster1Slave0BaseAddress \
                          + ( VrbSlotMin + VrbNum ) * A24VrbModuloAddress \
                          + Vrb_Control_AddrOffset
                          
    Vrb_Control_Value = 0x0000  # default: VRB in the readout crate but told not readout
    
    if ( VrbNum < VrbReadoutTot ) :     
        Vrb_Control_Value = 0x0003  # VRB is told to readout

    Vme_Write ( VmeAddress = Vrb_Control_VmeAddr, 
                   DataOut = Vrb_Control_Value )   


# VRB Reset
# -----------
#
# For these changes to take effect we must Reset the VRBs individually (type 0).
# The offset for this register is 0x0X003C
#

Vrb_Reset_AddrOffset = 0x3c

# emulation mode is disabled  for all the channels - VRB in slot 9

for VrbNum in range(VrbInCrateTot) :

    Vrb_Reset_VmeAddr = A32VIMaster1Slave0BaseAddress \
                          + ( VrbSlotMin + VrbNum ) * A24VrbModuloAddress \
                          + Vrb_Reset_AddrOffset

    Vme_Write ( VmeAddress = Vrb_Reset_VmeAddr, 
                   DataOut = 0x0000 )   
    

