/******************************************************************/

const char py_itc_version[]  = "V0.2" ;

/******************************************************************
/******************************************************************
// Python Interface to ITC routines
//
//  Created: 11-Oct-2004      Philippe Laurens
//
//  Modified: 
//
/******************************************************************
//     function      Python Usage
// 
//  - version        Message = py_itc.version() 
//
//                 Arguments:
//                      None
//
//                   Returns:
//                      string Message
//                              Includes version of this interface code 
//                              and version of bit3 API code
//                              e.g. "py_itc V1.0 python interface to bit3 API V2.4"
//
//  - new_client     HandleItcClient = py_itc.new_client ( TcpPortNum, HostName ) 
//
//                 Arguments:
//                      integer TcpPortNum
//                              Port Number of ITC Server to connect to
//                      string HostName
//                              The ip address of the host offering this ITC Server
//                   Returns:
//                      integer HandleItcClient
//                              Pointer to Itc Client object 
//                              to be used in subseqent calls to send_message, etc
//
//  - send_message   Status = py_itc.send_message ( HandleItcClient, Message ) 
//
//                 Arguments:
//                      integer HandleItcClient
//                              Pointer to Itc Client object 
//                              that was obtained from previous call to new_client
//                       string Message
//                              The ascii message to be sent
//                   Returns:
//                      integer Status 
//                              ITC status value from send_now
//                              0 = success
//                              1 = error  (e.g. connection broken) 
//
//  - get_reply      Status, Message = py_itc.get_reply ( HandleItcClient, TimeoutSeconds ) 
//
//                 Important:
//                              The maximum received message length is hardcoded to 1000 char
//                 Arguments:
//                      integer HandleItcClient
//                              Pointer to Itc Client object 
//                              that was obtained from previous call to new_client
//             optional integer TimeoutSeconds
//                              maximum waiting time for a message, in seconds
//                              default = 1 second
//                   Returns:
//                      integer Status 
//                              ITC status value to 
//                              0 = success
//                              1 = error  (e.g. timeout) 
//                      string  Message
//                              The ascii message received
//                              or "reply timeout" in case of timeout
//
/******************************************************************/

#include "Python.h"

/* only listen to the first 1000 char of reply */
#define     KiReplyMaxLen   1000

/******************************************************************/

#include "InterfaceItc.h"
#include "L1Cal_IIb_TCC/ListOfIPPortNumbers.h"

/******************************************************************/
/******************************************************************/

/******************************************************************/
/******************************************************************/
static PyObject*  version ( PyObject *self, PyObject *args )
{
    char            string[50] ;

    sprintf ( string, 
             "py_itc %s with ITC %s", 
              py_itc_version,
              itc_version() ) ;

    return Py_BuildValue("s", string ) ;
}

/******************************************************************/
/******************************************************************/
static PyObject*  new_client ( PyObject *self, PyObject *args )
{
    int         iIP_Port_Num = KiIP_Port_CoorToTcc ;     
    char*       Host = NULL ;
    int         pClient ;
 
    /* //\\// */

    /* retrieve python call arguments*/
    if (!PyArg_ParseTuple (args, "|is:new_client", &iIP_Port_Num, &Host ) )
    return NULL ;

    /* decode arguments */
    pClient = itc_new_client ( iIP_Port_Num, Host ) ;

    /* prepare return values */
    return Py_BuildValue ( "l", (long) pClient ) ;
}

/******************************************************************/
/******************************************************************/
static PyObject*  send_message ( PyObject *self, PyObject *args )
{
    int         pClient ;
    int         status ;
    char*       message ;

    /* //\\// */

    /* retrieve python call arguments*/
    if (!PyArg_ParseTuple (args, "is:send_message", &pClient, &message ) )
    return NULL ;

    /* decode arguments */
    status = itc_send_message ( pClient, message ) ;

    /* prepare return values */
    return Py_BuildValue ( "l", (long) status ) ;
}

/******************************************************************/
/******************************************************************/
static PyObject*  get_reply ( PyObject *self, PyObject *args )
{
    int         pClient ;
    int         timeout ;
    int         status ;
    char        reply[KiReplyMaxLen] ;

    /* //\\// */

    /* retrieve python call arguments*/
    timeout = 1 ; /* default one second */
    if (!PyArg_ParseTuple (args, "i|i:get_reply", &pClient, &timeout ) )
    return NULL ;

    /* decode arguments */
    status = itc_get_reply    ( pClient, &reply[0], KiReplyMaxLen, timeout ) ;

    /* prepare return values */
    return Py_BuildValue ( "(ls)", (long) status, reply ) ;
}

/******************************************************************/
/******************************************************************/
static PyObject*  del_client ( PyObject *self, PyObject *args )
{
    int         pClient ;
 
    /* //\\// */

    /* retrieve python call arguments*/
    if (!PyArg_ParseTuple (args, "i:get_reply", &pClient ) )
    return NULL ;

    /* decode arguments */
    itc_del_client ( pClient ) ;

    /* prepare return values */
    return Py_BuildValue ( "l", NULL ) ;
}

/******************************************************************/
/******************************************************************/
static PyMethodDef py_itc_methods[] =
{
	{"version",       version,           METH_VARARGS,  "version() doc string"},
	
    {"new_client",    new_client,        METH_VARARGS,  "new_client() doc string"},
	{"send_message",  send_message,      METH_VARARGS,  "send_message() doc string"},
	{"get_reply",     get_reply,         METH_VARARGS,  "get_reply() doc string"},
	{"del_client",    del_client,        METH_VARARGS,  "del_client() doc string"},

	{NULL, NULL, 0, NULL}        /* Sentinel */
};

/******************************************************************/
/******************************************************************/

/* DL_EXPORT(void) initpy_itc (void)
*/
void initpy_itc (void)
{

    Py_InitModule ( "py_itc", py_itc_methods ) ;

}

