// These are the low routines to read FPGA Config files

#include "L1Cal_IIb_TCC/StdAfx.h"

#include "L1Cal_IIb_TCC/GlobalConstants.h"
#include "L1Cal_IIb_TCC/CustomDataTypes.h"

#include                            "L1Cal_IIb_TCC/HandleSRecordFiles.h"

#include "L1Cal_IIb_TCC/DataManip.h"
#include "L1Cal_IIb_TCC/UtilsStrings.h"
#include "L1Cal_IIb_TCC/UtilsFiles.h"

// static means created only once throughout life of program (to avoid wasting time)
// *AND* internal linkage (to avoid clash between modules)
#include "L1Cal_IIb_TCC/HandleConsole.h"
static CConsoleAccess oConsole ;

/*----------------------------------------------------------------------------------*/
// Open an S-Record file on disk and transfer its content to an array  
/*----------------------------------------------------------------------------------*/
ESuccess Read_FPGA_Config_File ( const char* const      _sFileName,	
                                 unsigned char* const   _aubByteStream,
                                 int* const             _piNumBytesRead ) 
{
    ESuccess eReturnStatus ;
    ESuccess eSuccess ;

	FILE *hSRecFile ;
	char acFileName[_MAX_FNAME] ;
	char acMode_Read[]="r", acMode_Text[]="t" ;
	char acMode_Open[5]="", *pacMode_Open=acMode_Open ;

    int iLineNum, iSRecNum ;
    unsigned char acBuffer[133] ; 
    
    struct SRecord
	{	char acType[2] ;
		int  iLength ;
		unsigned char aucAddress [4] ;
		unsigned char aaucData[16][4] ;
		unsigned char aucCheckSum[4] ;
	}      xSRecord ;
	SRecord *pxSRecord = &xSRecord;

    int iFilledFields ;
	int iNumDataOnLine, iRelativeByteNum ;
	int iRunningAddress ;
	
    MultiAccessLong xRunningCheckSum ;
		
	EYesNo  ynFoundLastSRec  ;
	EYesNo  ynReadingHeaderComments ;

    char    cThisFileSRecType ;
    int     iSRecAddressLength ;
    char    acSRecContentFormat[62] ;
	
    //\\//

	//The functionj will return a success (True) only if it can open, read and parsed
	eReturnStatus   = Error ;
    *_piNumBytesRead = 0 ; 

    // locate the S-Record file 
    // using any embedded environment variable 
	eSuccess = LocateFile ( _sFileName, acFileName ) ;

    if ( eSuccess == Error )
		goto Error_Open ;

    if ( strcmp ( acFileName, "" ) == 0 )
    {
        oConsole.ErrorLine ( (CString) "%% Could not locate file <" 
                                      + _sFileName + ">" ) ;
		goto Error_Open ;
    }

	// open the file (and check for success)
	//set Read only mode, and text type flags
	strcat ( acMode_Open, acMode_Read ) ;
	strcat ( acMode_Open, acMode_Text ) ;
	hSRecFile = fopen ( acFileName, acMode_Open ) ;
	if (hSRecFile == NULL) 
	{
		oConsole.ErrorLine ( (CString) "_ could not open the file " + acFileName ) ;
		goto Error_Open ;
	}
//	else
//		oConsole.WriteLine ( (CString) "_ " + acFileName + " File opened" ) ;

	
	// read the file 
	// but it MAY contain comment lines at the top
	// read untill you reach a line starting with "S" before decoding S-records
	ynFoundLastSRec         = No ;
	ynReadingHeaderComments = Yes ;
	iLineNum = 0 ;
	iSRecNum = 0 ;
	iRunningAddress = 0 ;
	iNumDataOnLine = 16 ;

Read_Next_Record: 
    
    // read first two characters of next record to decide what it is, e.g. "S0"
	iLineNum++ ;
	iFilledFields = fscanf ( hSRecFile, 
		                     "%c%c", 
							 &xSRecord.acType[0],
                             &xSRecord.acType[1] ) ;

    // Check read success
	if ( iFilledFields != 2 ) 
	{
        oConsole.ErrorLine ( (CString) "_ Failure parsing characters at line #" 
                                     + I_to_A ( iLineNum ) ) ;
		goto close_file ;
	}
    

    // handle the Header Comments
    if ( ynReadingHeaderComments == Yes ) 
    {
	    // Check for end of comment section
	    if ( xSRecord.acType[0] == 'S' ) 
        {// We have found the first S-Record
            ynReadingHeaderComments = No ;
            // we can adjust for differences between S1 and S2 records
	        switch ( xSRecord.acType[1] )
	        { 
		        case '1':
			        cThisFileSRecType = '1' ;
                    iSRecAddressLength = 2 ;
        	        strcpy ( acSRecContentFormat, 
                            "%2x%4x%2x%2x%2x%2x%2x%2x%2x%2x%2x%2x%2x%2x%2x%2x%2x%2x%2x%[\n]" ) ;
			        break ;
	        
        		case '2':
			        cThisFileSRecType = '2' ;
                    iSRecAddressLength = 3 ;
        	        strcpy ( acSRecContentFormat, 
                            "%2x%6x%2x%2x%2x%2x%2x%2x%2x%2x%2x%2x%2x%2x%2x%2x%2x%2x%2x%[\n]" ) ;
        			break ;
	        	        
		        default :
                    oConsole.ErrorLine ( (CString) "_ First S-Record is of Unexpected Type 'S" 
		                                         + xSRecord.acType[1]
                                                 + "' at line #" 
                                                 + I_to_A ( iLineNum ) ) ;
			        goto close_file ;
	        }
        }
        else
	    {// another comment line, since it didn't start with 'S'
		            
            // Now read/discard the rest of the line
            // except if there was only 0 or 1 char on the line
            if ( ( xSRecord.acType[0] != '\n' ) && ( xSRecord.acType[1] != '\n' ) ) 
                fgets ( (char *) &acBuffer[0], 132, hSRecFile )  ; 

            goto Read_Next_Record ;
        }
    }
    else
    { // We are no longer in comments header, this must be an S-Record
	    if ( xSRecord.acType[0] != 'S' ) 
        {// error: no longer in header comment section, and line doesn't start with 'S'
            oConsole.ErrorLine ( (CString) "_ No 'S' found in first column at line #" 
                                         + I_to_A ( iLineNum ) ) ;
	        goto close_file ;
        }
    }

    
    // From now on, this is parsed as an S-Record, 
	iSRecNum++ ;

    // Check it is still the same type of S-Record as first one
	if ( xSRecord.acType[1] != cThisFileSRecType )
	{ 
		if ( ( xSRecord.acType[1] == '8' ) || ( xSRecord.acType[1] == '9' ) )
        {
//            oConsole.WriteLine ( (CString) "_ Found end record 'S"
//                                         + xSRecord.acType[1]
//                                         + "' (content not parsed) at line #" 
//                                         + I_to_A ( iLineNum ) ) ;
//            oConsole.WriteLine ( (CString) "_ Stop Parsing S-Record Byte Stream at line #" 
//                                         + I_to_A ( iLineNum ) ) ;
			goto Success ; // claim success without actually parsing S9 record or beyond			
        }
		else
        {// No other type of record allowed besides S1/S2, or S8/S9
            oConsole.ErrorLine ( (CString) "_ Unexpected switch in S-Record type from 'S"
                                         + cThisFileSRecType
                                         + "' to 'S"
                                         + xSRecord.acType[1]
                                         + "' at line #" 
                                         + I_to_A ( iLineNum ) ) ;
			goto close_file ;
        }
	}


  	// read rest of S-Record, i.e. all Hex codes after the 'S1' or 'S2' type field
	iFilledFields = fscanf ( hSRecFile, 
							 acSRecContentFormat,
							 &xSRecord.iLength,
							 (unsigned int *) &xSRecord.aucAddress,
							 (unsigned int *) &xSRecord.aaucData[0],
							 (unsigned int *) &xSRecord.aaucData[1],
							 (unsigned int *) &xSRecord.aaucData[2],
							 (unsigned int *) &xSRecord.aaucData[3],
							 (unsigned int *) &xSRecord.aaucData[4],
							 (unsigned int *) &xSRecord.aaucData[5],
							 (unsigned int *) &xSRecord.aaucData[6],
							 (unsigned int *) &xSRecord.aaucData[7],
							 (unsigned int *) &xSRecord.aaucData[8],
							 (unsigned int *) &xSRecord.aaucData[9],
							 (unsigned int *) &xSRecord.aaucData[10],
							 (unsigned int *) &xSRecord.aaucData[11],
							 (unsigned int *) &xSRecord.aaucData[12],
							 (unsigned int *) &xSRecord.aaucData[13],
							 (unsigned int *) &xSRecord.aaucData[14],
							 (unsigned int *) &xSRecord.aaucData[15],
							 (unsigned int *) &xSRecord.aucCheckSum,
							 &acBuffer[0] ) ; //acBuffer will get the end of line, or any unexpected overflow


    // First, check we weren't supposed to be done
	if ( ynFoundLastSRec == Yes )
	{
        oConsole.ErrorLine ( (CString) "_ Unexpected S-Record found after short (assumed to be last) one"
                                     + " at line #" 
                                     + I_to_A ( iLineNum ) ) ;
		goto close_file ;
	}

    // Check fscanf found at least a length, address, 1 byte, checksum, and <CR> 
	if ( iFilledFields < 5 )
	{
        oConsole.ErrorLine ( (CString) "_ Not enough characters for a valid S-record at line #" 
                                     + I_to_A ( iLineNum ) ) ;
		goto close_file ;
	}
	
    // Check S-Record record length and adjust if it is not a full length record
    // with 2 or 3 bytes of address, 16 bytes of data, and one byte checksum
	if ( xSRecord.iLength < iSRecAddressLength + 16 + 1 )
	{
		//adjust excpected number of data bytes, and copy CheckSum to natural place
		iNumDataOnLine = xSRecord.iLength - iSRecAddressLength - 1 ;
		xSRecord.aucCheckSum[0] = xSRecord.aaucData[iNumDataOnLine][0] ;
        //remember we found a shorter (i.e. last) record, so next record must be S9
        ynFoundLastSRec = Yes ;
	}    
    // check number of fields successfully read, and check for an ending linefeed
	else if ( ( iFilledFields != 20) || ( acBuffer[0] != '\n' ) )
	{
        oConsole.ErrorLine ( (CString) "_ S-Record format or line length problem at line #" 
                                     + I_to_A ( iLineNum ) ) ;
		goto close_file ;
	}

	// we have kept track of the (fake) destination addresses and it should match
	if ( *(int *) xSRecord.aucAddress != iRunningAddress )
	{
        oConsole.ErrorLine ( (CString) "_ S-Record Address mismatch at line #" 
                                     + I_to_A ( iLineNum ) ) ;
		goto close_file ;
	}

	// re-compute the line's S-Record checksum, starting with length and address fields
	xRunningCheckSum.ulLong = 0 ;
	xRunningCheckSum.ulLong += xSRecord.iLength ;
    int iSRecAddressByte ;
    for ( iSRecAddressByte = 0 ; 
          iSRecAddressByte < iSRecAddressLength ;
          iSRecAddressByte++ ) 
	xRunningCheckSum.ulLong += xSRecord.aucAddress[iSRecAddressByte] ; 

	// transfer each byte and update CheckSum, and running address
	for ( iRelativeByteNum = 0 ;
		  iRelativeByteNum < iNumDataOnLine ; 
		  iRelativeByteNum++, iRunningAddress++ ) 
	{ 
        // keep an eye on the running address to make sure we don't fall off the end
        // of the space allocated to the ByteStream array
        if ( iRunningAddress >= diFPGA_Config_Byte_Size_Max ) 
        { 
            oConsole.ErrorLine ( (CString) "_ Byte Stream longer than expected Maximum of"
                                         + I_to_A ( diFPGA_Config_Byte_Size_Max )
                                         + " at line #" 
                                         + I_to_A ( iLineNum ) ) ;
		    goto close_file ;
        }
		xRunningCheckSum.ulLong += xSRecord.aaucData[iRelativeByteNum][0] ;
		_aubByteStream[iRunningAddress] = xSRecord.aaucData[iRelativeByteNum][0] ;
    }
	  
	// Record's CheckSum is one's complement of RunningCheckSum 
	if ( xSRecord.aucCheckSum[0] + xRunningCheckSum.ByByte.ubByte_0 != 0xff )
	{
        oConsole.ErrorLine ( (CString) "_ S-Record Checksum Mismatch at line #" 
                                     + I_to_A ( iLineNum ) ) ;
		goto close_file ;
	}

goto Read_Next_Record ;

Success:
    // compute number of bytes read using last running address 
    // which was automatically incremented one last time after exiting the last loop
    *_piNumBytesRead = iRunningAddress ; 

    //now check the byte stream length read to expected set of possible length
         if ( *_piNumBytesRead == diFPGA_Config_Byte_Size_XC2V1000_t3 )
    { 
        oConsole.WriteLine ( (CString) "_ Byte Stream Length consistent with XC2V1000(t2/t3) = " 
                                     + I_to_A ( *_piNumBytesRead ) 
                                     + " bytes" ) ;
    } 
    else if ( *_piNumBytesRead == diFPGA_Config_Byte_Size_XC2V1000_t4 )
    { 
        oConsole.WriteLine ( (CString) "_ Byte Stream Length consistent with XC2V1000(t4) = " 
                                     + I_to_A ( *_piNumBytesRead ) 
                                     + " bytes" ) ;
    } 
    else if ( *_piNumBytesRead == diFPGA_Config_Byte_Size_XC2V1000_max )
    { 
        oConsole.WriteLine ( (CString) "_ Byte Stream Length consistent with XC2V1000(max) = " 
                                     + I_to_A ( *_piNumBytesRead ) 
                                     + " bytes" ) ;
    } 
    else 
    {
        oConsole.ErrorLine ( (CString) "_ Byte Stream Length inconsistent with expected sizes = " 
                                     + I_to_A ( *_piNumBytesRead ) 
                                     + " bytes" ) ;
        oConsole.ErrorLine ( (CString) "_ ...but continuing" ) ;
    }

    // We only get here if we were successfull
    eReturnStatus = Ok ;

close_file: 
	// now close the file
/*    oConsole.WriteLine ( (CString) "_ Closing File after Reading " 
                                 + I_to_A ( iLineNum ) 
                                 + " Lines and finding "
                                 + I_to_A ( iSRecNum )
                                 + " S-Records" ) ;
*/
	if ( fclose (hSRecFile) ) 
        oConsole.ErrorLine ( (CString) "_ The file didn't close correctly" ) ;

Error_Open:
	return eReturnStatus ;
}

