#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <unistd.h>
#include "tabgab/util.h"
#include "tabgab/vmescl.h"
#include "tabgab/tab.h"

//convenience functions:
//
//
// Setting up the l2 buffer header is done in two parts.
//
// The firmware version refers to the *entire bundle* of TAB/GAB firmware
// and is set over VME immediately after the chip is configured.
//
// The second step is two set the remainder of the header according to the
// level 2 convention, e.g. l2dtype = 192 + module
//

int global_setup_level2_header_firmware_version(int module, int major, int minor){
    TRY (global_raw_write (module, 4, major, minor    )); //firmware major, minor
    return EXIT_SUCCESS;
}

int global_setup_level2_header(int module){
    TRY (global_raw_write (module, 0, 3,   1          )); //hlength,         objects
    TRY (global_raw_write (module, 1, 0x21, 0x5d      )); //format,          olength
    TRY (global_raw_write (module, 2, 0,   192+module )); //                 l2dtype
    //  (set earlier)                                     // firmware major, minor
    TRY (global_raw_write (module, 5, 0,   3          )); //                 procbit
    return EXIT_SUCCESS;
}

int global_setup_energy_params(int module){

    TRY (global_energy_write(module, 0, GLOBAL_SUM_THRESHOLD));
        
    int indphi = 0;
    unsigned long value;
    for(int j=1; j< 9;j++){
	indphi = j/2;
	if(j%2 == 0) value = cosine_lookup(4*module+(j-1)/2 );
	else         value = sine_lookup(4*module+(j/2) );
	TRY (global_energy_write(module, j, value));
    }
    return EXIT_SUCCESS;
}














int global_raw_mem_read
    (unsigned long module, unsigned long address, unsigned long *value)
{
    TRY (check_tab_or_gab(module));

    if (address >= 256) {
	error("The address must be smaller than 256");
	return EXIT_FAILURE;
    }     

    return vmescl_read_serial(module, TAB_CHIP_GLOBAL, GLOBAL_GROUP_READ_RAW_MEM, 
			   address, 0, value);
}

int global_alg_mem_read(unsigned long module, 
			   unsigned long event, unsigned long channel, 
			   unsigned long *value) 
{ 
    
    unsigned long seradr;
    seradr = (event << NUM_GLOB_GAB_CHBITS) | channel; 
    
    TRY (check_tab_or_gab(module));
    
    if (event >= NUM_GLOB_GAB_EVENTS) {
	error("The event number must be smaller than %d", NUM_GLOB_GAB_EVENTS);
	return EXIT_FAILURE;
    } 

    if (channel >= NUM_GLOB_GAB_CHANNEL) {
	error("The channel number must be smaller than %d", NUM_GLOB_GAB_CHANNEL);
	return EXIT_FAILURE;
    } 
  
    return vmescl_read_serial(module, TAB_CHIP_GLOBAL, 
			  GLOBAL_GROUP_READ_ALG_MEM, 
			  seradr, 0, value); 
} 


/* writes the global parameters */

#define STRINGLEN 200

#define MAX_GLOBAL_DELAY 255
#define MAX_GLOBAL_ENERGY_PARAMETER 8
#define MIN_GLOBAL_ENERGY_PARAMETER 0
#define MAX_GLOBAL_RAW_PARAMETER 7
#define MIN_GLOBAL_RAW_PARAMETER 0


int global_delay_write(unsigned long module, unsigned long delay)
{
    TRY( check_tab_or_gab (module)                                                );
    if (delay > MAX_GLOBAL_DELAY) {
	error("The maximum delay value is %d", MAX_GLOBAL_DELAY);
	return EXIT_FAILURE;
    }
    
    return vmescl_write_serial(module, TAB_CHIP_GLOBAL, GLOBAL_GROUP_WRITE_SCL_DELAY, 0, delay);
}


int global_energy_write(unsigned long module, unsigned long seradr, unsigned long value)
{
    TRY( check_tab_or_gab (module)                                                );
  
    if (seradr > MAX_GLOBAL_ENERGY_PARAMETER || 
	seradr < MIN_GLOBAL_ENERGY_PARAMETER) {
	error("The seradr value is between %d and %d", 
	      MIN_GLOBAL_ENERGY_PARAMETER,
	      MAX_GLOBAL_ENERGY_PARAMETER);
	return EXIT_FAILURE;
    }
  
    return vmescl_write_serial(module, TAB_CHIP_GLOBAL, GLOBAL_GROUP_WRITE_E_PAR, seradr, value);  
}


int global_raw_write(unsigned long module, 
			    unsigned long seradr, unsigned long highvalue,
			    unsigned long lowvalue)
{
    TRY( check_tab(module));
    
    unsigned long value;
    value = ( highvalue << 8 ) | lowvalue;
        
    if (seradr > MAX_GLOBAL_RAW_PARAMETER || 
	seradr < MIN_GLOBAL_RAW_PARAMETER) {
	error("The seradr value is between %d and %d", 
	      MAX_GLOBAL_RAW_PARAMETER,
	      MIN_GLOBAL_RAW_PARAMETER);
	return EXIT_FAILURE;
    } 
    
    return vmescl_write_serial(module, TAB_CHIP_GLOBAL, GLOBAL_GROUP_WRITE_RAW_PAR, seradr, value);  
}
    



/*

 * And now the function that actually reads an input file and calls
 * the above functions. The format of the file is a sequence of lines
 * each interpreted as:

del 'value'
(sets the timing delay parameter)

energy 'seradr' 'serval'
(sets the energy parameters)

raw 'seradr' 'datahigh' 'datalow'
(sets the raw parameters)
*/

int global_set_pars_from_file(unsigned long module, char *filename)
{
    int retVal;
    FILE *fp;
    char s[STRINGLEN];
    char type[20];
    unsigned long second, third, fourth;
    
    if((fp = fopen(filename,"r")) == NULL) {
	error("Cannot open %s", filename);
	return EXIT_FAILURE;
    }
        
    while(fgets(s, STRINGLEN, fp)) {
	retVal = sscanf(s,"%19s %lx %lx %lx", type, &second, &third, &fourth);
	if (retVal == 2 && strcmp(type, "del") == 0) {
	    retVal = global_delay_write(module, second);
	    if (retVal != EXIT_SUCCESS) {
		return retVal;
	    }
	} else if (retVal == 3 && strcmp(type, "energy") == 0) {
	    retVal = global_energy_write(module, second, third);
	    if (retVal != EXIT_SUCCESS) {
		return retVal;
	    }
	} else if (retVal == 4 && strcmp(type, "raw") == 0) {
	    retVal = global_raw_write(module, second, third, fourth);
	    if (retVal != EXIT_SUCCESS) {
		return retVal;
	    }
	} else if (retVal != 0) {
	    // the line must be a comment
	    if (type[0] != '#') {
		error("global parameter file has a malformed line");
		return EXIT_FAILURE;
	    }
	}
    }
    fclose(fp);
    return EXIT_SUCCESS;
}

int global_track_mem_read(unsigned long module, unsigned long line, unsigned long *value)
{  
    TRY (check_tab_or_gab(module));
    
    if (line >= NUM_GLOB_TRACK_EVENTS * NUM_GLOB_TRACK_LINES_PER_EVENT) 
    {	
	error("The line number must be smaller than %d",
	      NUM_GLOB_TRACK_EVENTS * NUM_GLOB_TRACK_LINES_PER_EVENT);
	return EXIT_FAILURE;
    }
    
    return vmescl_read_serial(module, TAB_CHIP_GLOBAL, GLOBAL_GROUP_READ_TRACK_MEM, line, 0, value);
}

