#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <unistd.h>
#include "tabgab/util.h"
#include "tabgab/vme.h"
#include "tabgab/vmescl.h"
#include "tabgab/tab.h"
#include "tabgab/gab.h"

//control size of strings for determining firmware filenames:
#define BUFSIZE 10
#define FILENAME_SIZE 200

//how long to wait between cyclone writes and reads...
#define WAIT_TIME 100
#define LONG_WAIT_TIME 100000

//prototypes for locally used functions:
int usleep(unsigned int microseconds);
int stratix_do_preconfig    (unsigned long module, char type); 
int stratix_parse_file      (char filename[], unsigned long module, char type);
int stratix_do_postconfig   (unsigned long module, char type);
int stratix_pll_init        (unsigned long module, unsigned long stratix); 
int stratix_reset           (unsigned long module, unsigned long chipmask);

// read the status in a stratix chip (regular or global)
int stratix_status_read(unsigned long module, unsigned long chip, unsigned long *data)
{
    TRY(check_tab_or_gab(module));
    TRY(check_module_and_chip(module,chip));

    if (is_tab(module)){
	if (is_global_stratix(module,chip)){
	    return vmescl_read_serial(module, chip, GLOBAL_GROUP_READ_STATUS, 0, 0, data);
	} else {
	    return vmescl_read_serial(module, chip, SWA_GROUP_STATUS, 
				   SWA_GROUP_STATUS_SERADR_READ, 0, data);
	}
    } else {
	if (is_global_stratix(module,chip)){
	    return vmescl_read_serial(module, chip, S30_GROUP_STATUS_READ, 0, 0, data);
	} else {
	    return vmescl_read_serial(module, chip, SWA_GROUP_STATUS, 
				   SWA_GROUP_STATUS_SERADR_READ, 0, data);
	}
    }
}

// print the status of a stratix chip (regular or global)
int stratix_status_print(unsigned long module, unsigned long chip, unsigned long data)
{
    TRY(check_tab_or_gab(module));
    TRY(check_module_and_chip(module,chip));
    
    if (is_tab(module)){
	if (is_global_stratix(module,chip)){
	    info("_tab global status_");
	    info("MODE:           %d",(data&GLOBAL_STATUS_MODE)         !=0);
	    info("CP90 LOCK:      %d",(data&GLOBAL_STATUS_CP90_LOCK)    !=0);
	    info("CP53 LOCK:      %d",(data&GLOBAL_STATUS_CP53_LOCK)    !=0);
	    info("RUN:            %d",(data&GLOBAL_STATUS_RUN)          !=0);
	} else {
	    info("_tab swa status, chip %d_", (int) chip);	
	    info("PERR0:          %d",(data&SWA_STATUS_PERR0)           !=0);
	    info("PERR1:          %d",(data&SWA_STATUS_PERR1)           !=0);
	    info("PERR2:          %d",(data&SWA_STATUS_PERR2)           !=0);
	    info("SYNCERR0:       %d",(data&SWA_STATUS_SYNCERR0)        !=0);
	    info("SYNCERR1:       %d",(data&SWA_STATUS_SYNCERR1)        !=0);
	    info("SYNCERR2:       %d",(data&SWA_STATUS_SYNCERR2)        !=0);
	    info("BCERR0:         %d",(data&SWA_STATUS_BCERR0)          !=0);
	    info("BCERR1:         %d",(data&SWA_STATUS_BCERR1)          !=0);
	    info("BCERR2:         %d",(data&SWA_STATUS_BCERR2)          !=0);
	    info("PLL_LOCK_ERR:   %d",(data&SWA_STATUS_PLL_LOCK_ERR)    !=0);
	    info("MODE_FF         %d",(data&SWA_STATUS_MODE_FF)         !=0);
	    info("LFSR1           %d",(data&SWA_STATUS_LFSR1)           !=0);
	    info("LFSR2           %d",(data&SWA_STATUS_LFSR2)           !=0);
	    info("LFSR3           %d",(data&SWA_STATUS_LFSR3)           !=0);
	    info("EXPERT 1:       %d",(data&SWA_STATUS_EXPERT1)         !=0);
	    info("EXPERT 2:       %d",(data&SWA_STATUS_EXPERT2)         !=0);
	    info("EXPERT 1: currently sync align error");
	    info("EXPERT 2: currently pll locked");	    	    
	}
    } else {
	if (is_global_stratix(module,chip)){
	    info("_gab s30 status_");
	    //firmware:
	    //status(0) <= init;
	    //status(1) <= sclrun;
	    //status(2) <= locked;
	    //status(3) <= sclerror;
	    //status(4) <= sclspare;
	    info("INIT:           %d",(data & S30_STATUS_INIT      ) !=0);
	    info("SCL RUN:        %d",(data & S30_STATUS_SCL_RUN   ) !=0);
	    info("LOCKED:         %d",(data & S30_STATUS_LOCKED    ) !=0);
	    info("SCL ERROR:      %d",(data & S30_STATUS_SCL_ERROR ) !=0);
	    info("SCL SPARE:      %d",(data & S30_STATUS_SCL_SPARE ) !=0);
	} else {
	    //firmware:
	    //data(0)  => pulse,
	    //data(1)  => nl,
	    //data(2)  => parity_error(0),
	    //data(3)  => parity_error(1),
	    //data(4)  => inst_parity_error(0),
	    //data(5)  => inst_parity_error(1),
	    //data(6)  => sync_error,
	    //data(7)  => bc_error,
	    //data(8)  => valid_error,
	    //data(9)  => sclbc_error,
	    //data(10) => sclturn_error,
	    //data(11) => pll_lock_error,  
	    info("_gab lvds status chip %d_",(int) chip);	
	    info("PULSE:          %d",(data&(1<<0)  )        !=0);
	    info("NL:             %d",(data&(1<<1)  )        !=0);
	    info("PERR0:          %d",(data&(1<<2)  )        !=0);
	    info("PERR1:          %d",(data&(1<<3)  )        !=0);
	    info("INST PERR0:     %d",(data&(1<<4)  )        !=0);
	    info("INST PERR1:     %d",(data&(1<<5)  )        !=0);	    
	    info("SYNC ERR:       %d",(data&(1<<6)  )        !=0);
	    info("BC ERR:         %d",(data&(1<<7)  )        !=0);
	    info("VALID ERR:      %d",(data&(1<<8)  )        !=0);
	    info("SCL BC ERR:     %d",(data&(1<<9)  )        !=0);
	    info("SCL TURN ERROR: %d",(data&(1<<10) )        !=0);
	    info("PLL LOCK ERROR: %d",(data&(1<<11) )        !=0);
	}		
    }
    return EXIT_SUCCESS;
}


// writes to the stratix status register
int stratix_status_write(unsigned long module, unsigned long chip, unsigned long data)
{
    TRY(check_tab_or_gab(module));
    TRY(check_module_and_chip(module,chip));
    
    return vmescl_write_serial(module, chip, SWA_GROUP_STATUS, SWA_GROUP_STATUS_SERADR_WRITE, data);
}

//configure all chips in a TAB or GAB using files of a specifie
int stratix_configure         ( unsigned long module, const char * dir, const char * pkg_version)
{
    const char tab_base[] = "tab_stratix_";
    const char gab_base[] = "gab_stratix_";
    const char * filebase;

    char filename[FILENAME_SIZE];
    unsigned long count;
    unsigned long mask = 1;

    unsigned long gab, nreg;  //number of regular stratixes: tab=10, gab=4;
    
    int major, minor;    
    if (sscanf(pkg_version,"version_%d.%d",&major,&minor)==2){
	info("--> package version is %d.%d, will set firmware version after global configure.",major,minor);	
    } else {
	//set to generic testing version, which at least for now is 1.1:
	major = 1;
	minor = 1;	
	info("--> package version %s is not of format version_X.Y, i.e. a test version.",pkg_version);
	info("    Will set firmware version to %d.%d after global configure.",major,minor);	
    }
		
    TRY (check_tab_or_gab(module));
    if (module == MOD_GAB){
	nreg = 4;
	gab  = 1;
	filebase = gab_base;
    } else {
	nreg = 10;
	gab  = 0;
	filebase = tab_base;
    }
        
    TRY (cyclone_status_set(module, CYCLONE_STATUS_MASK_CONF_MUX_A | CYCLONE_STATUS_MASK_CONF_MUX_B));

    info("--> doing preconfig type a first...");
    TRY (stratix_do_preconfig(module, 'a'));
    
    for (count = 0; count < nreg; count++) {
	info("--> programing regular stratix:  module %d, chip %d", (int) module, (int) count);
	sprintf(filename, "%s/%s/%s%02d.ttf",dir,pkg_version,filebase,(int) count);	    	
	TRY (stratix_parse_file(filename, module, 'a'));
    }    
    info("    --> doing postconfig type a.");
    TRY (stratix_do_postconfig(module, 'a'));    
    info("--> done programing all regular stratix chips.");

    info("--> programing global stratix chip");
    info("--> doing preconfig type b first...");	    
    TRY(stratix_do_preconfig(module, 'b'));
    sprintf(filename, "%s/%s/%s%02d.ttf",dir,pkg_version,filebase, (int) count);	    	    	
    TRY(stratix_parse_file(filename, module, 'b'));
    info("    --> doing postconfig type b.");
    TRY(stratix_do_postconfig(module, 'b'));
    info("--> done programing global stratix chip %d",(int) count);
    
    TRY (cyclone_status_clear(module, CYCLONE_STATUS_MASK_CONF_MUX_A | CYCLONE_STATUS_MASK_CONF_MUX_B));
    
    TRY (stratix_reset(module, 0xfff));
        
    if (is_tab(module)){
	info("--> setting package firmware version to %d.%d in TAB global...",major,minor);
	global_setup_level2_header_firmware_version(module, major, minor);
	info("done");
    } else {
	info("--> setting package firmware version to %d.%d in GAB S30...",major,minor);
	s30_setup_level2_header_firmware_version(major, minor);
	info("done");
    }    
    return EXIT_SUCCESS;
}

/* after every how many bytes sent should nstatus 
 * and conf_done be checked? 
 * MUST BE A PWR OF 2 !!!!!!!  */
#define CHECK_STATUS_FREQ 1048576


int stratix_parse_file(char filename[], unsigned long module, char type) 
{  	       
    /* this is a local helper function */
    char number[BUFSIZE];
    int charread;
    int i = 0;
    unsigned bytecount = 0;
    unsigned mask = CHECK_STATUS_FREQ - 1;
    unsigned long nstatusmask;
    unsigned long confdonemask;
    unsigned long status;
    FILE *fp;
    unsigned long byteUpper = 0;	/* even or odd byte */
    unsigned long writeVal = 0;
    
    if (type == 'a' || type == 'A') {
	nstatusmask = CYCLONE_STATUS_MASK_NSTATUS_A;
	confdonemask = CYCLONE_STATUS_MASK_CONF_DONE_A;
    } else if (type == 'b' || type == 'B') {
	nstatusmask = CYCLONE_STATUS_MASK_NSTATUS_B;
	confdonemask = CYCLONE_STATUS_MASK_CONF_DONE_B;
    } else {
	error("Type needs to be 'a' or 'b'");
	return EXIT_FAILURE;
    }
    
    info("    ...downloading %s to module %d", filename, (int) module);
    fflush(stdout);
    
    if ((fp = fopen(filename, "r")) == NULL) {
	error("Couldn't open file: %s", filename);
	return EXIT_FAILURE;
    }
    while ((charread = getc(fp)) != EOF) {
	if (isspace(charread)) {
	    /* ignore */
	    ;
	} else if (charread == ',') {
	    if ((bytecount++ & mask) == mask) {
		/* do a status check */
		TRY (cyclone_status_read(module, &status));
		
		if (~status & nstatusmask) {	
		    /* nstatus is low */
		    error("nStatus %c went low", type);
		    return EXIT_FAILURE;
		}
		
		if (status & confdonemask) {	
		    /* conf done is high */
		    error("conf done %c went high before the firmware download completed",
			  type);
		    return EXIT_FAILURE;
		}
	    }	
	    number[i] = '\0';
	    i = 0;
	    if (byteUpper) {
		/* is the second of two bytes */
		writeVal |= (strtoul(number, NULL, 10) << 8);
		
		TRY (cyclone_write_conf_half_word(module, type, writeVal));
		
		byteUpper = 0;		/* next one will be lower */
	    } else {
		/* is the first of a pair of bytes */
		writeVal = strtoul(number, NULL, 10);
		byteUpper = 1;		/* next one will be upper */
	    }      
	}
	else if (isdigit(charread)) {
	    number[i++] = charread;
	}
	else {
	    error("File %s is of incorrect format", filename);
	    return EXIT_FAILURE;
	}
    }
    /* now do that last one */
    number[i] = '\0';
    info("    ...done:  type is %c, last byte is %s, number read is %u, byteUpper is %lu.", 
	   type, number, bytecount, byteUpper); 
    if (byteUpper) {
	/* last one is the second of a pair */
	writeVal |= (strtoul(number, NULL, 10) << 8);
	return cyclone_write_conf_half_word(module, type, writeVal);
    } else {
	/* do old style single transaction */
	return cyclone_write_conf_byte(module, type, strtoul(number, NULL, 10));
    }
}

int stratix_do_preconfig(unsigned long module, char type) 		 
{				
    /* do the nconfig toggle, nstatus, and all that */
    unsigned long nconfigmask;
    unsigned long nstatusmask;
    unsigned long confdonemask;
    unsigned long status;
    unsigned long serouten;
    
    if (type == 'a' || type == 'A') {
	nconfigmask  = CYCLONE_STATUS_MASK_NCONFIG_A;
	nstatusmask  = CYCLONE_STATUS_MASK_NSTATUS_A;
	confdonemask = CYCLONE_STATUS_MASK_CONF_DONE_A;
	serouten     = CYCLONE_STATUS_MASK_SEROUT_EN_A;
    } else if (type == 'b' || type == 'B') {
	nconfigmask  = CYCLONE_STATUS_MASK_NCONFIG_B;
	nstatusmask  = CYCLONE_STATUS_MASK_NSTATUS_B;
	confdonemask = CYCLONE_STATUS_MASK_CONF_DONE_B;
	serouten     = CYCLONE_STATUS_MASK_SEROUT_EN_B;
    } else {
	error("Type needs to be 'a' or 'b'");
	return EXIT_FAILURE;
    }
  
    /* disable the serouten so that if the stratixes were already programmed,
     * their output doesn't corrupt the VME/SCL to cyclone communication when 
     * they are configuring 
     */    
    TRY (cyclone_status_clear(module, serouten));
    
    /* make nconfig go low */
    TRY (cyclone_status_clear(module, nconfigmask));    
    usleep(WAIT_TIME);
    TRY (cyclone_status_read(module, &status));    
    info("    ...made nconfig go low (type %c)",type);
        
    if (status & nstatusmask) {	
	error("preconfig:  nStatus %c didn't go to low after nConfig went to low", type);
	return EXIT_FAILURE;
    }
    if (status & confdonemask) {	
	error("preconfig:  conf done %c didn't go to low after nConfig went to low", type);
	return EXIT_FAILURE;
    }

    /* make nconfig go high */
    TRY (cyclone_status_set(module, nconfigmask));
    usleep(LONG_WAIT_TIME);
    TRY (cyclone_status_read(module, &status));    
    info("    ...made nconfig go high (type %c)",type);

    if (~status & nstatusmask) {	
	error("preconfig: nStatus %c didn't go to high after nConfig went to high", type);
	return EXIT_FAILURE;
    }
    if (status & confdonemask) {	
	error("preconfig: conf done %c went to high before any firmware was downloaded", type);
	return EXIT_FAILURE;
    }

    return EXIT_SUCCESS;
}

#define NUM_EXTRA_DCLK 137

int stratix_do_postconfig(unsigned long module, char type) 		  
{				
    unsigned long nstatusmask;
    unsigned long confdonemask;
    unsigned long status;
    unsigned long serouten;
    int i = 0;

    if (type == 'a' || type == 'A') {
	nstatusmask  = CYCLONE_STATUS_MASK_NSTATUS_A;
	confdonemask = CYCLONE_STATUS_MASK_CONF_DONE_A;
	serouten     = CYCLONE_STATUS_MASK_SEROUT_EN_A;
    } else if (type == 'b' || type == 'B') {
	nstatusmask  = CYCLONE_STATUS_MASK_NSTATUS_B;
	confdonemask = CYCLONE_STATUS_MASK_CONF_DONE_B;
	serouten     = CYCLONE_STATUS_MASK_SEROUT_EN_B;
    } else {
	error("Type needs to be 'a' or 'b'");
	return EXIT_FAILURE;
    }
    
    TRY (cyclone_status_read(module, &status));

    if (~status & nstatusmask) {	
	error("postconfig:  nStatus %c was low after configuration ", type);
	return EXIT_FAILURE;
    }
    if (~status & confdonemask) {	
	error("postconfig:  conf done %c did not go to high after configuration", type);
	return EXIT_FAILURE;
    }
    for (i = 0; i < NUM_EXTRA_DCLK; i++) {
	TRY (cyclone_write_conf_byte(module, type, 0));
    }
    return cyclone_status_set(module, serouten);
}

// reset all the stratixes:
int stratix_reset(unsigned long module, unsigned long chipmask)
{
    unsigned long value;
    unsigned long mask = 1;
    unsigned long count;
    unsigned long nreg;
        
    TRY (check_tab_or_gab(module));
    
    /* reset the scl state status (i.e. whether in run or not) */
    
    TRY (vmescl_generate_mrst());
    TRY (cyclone_status_set   (module, CYCLONE_STATUS_MASK_MRST_A | CYCLONE_STATUS_MASK_MRST_B));
    TRY (cyclone_status_clear (module, CYCLONE_STATUS_MASK_MRST_A | CYCLONE_STATUS_MASK_MRST_B));
    TRY (cyclone_status_set   (module, CYCLONE_STATUS_MASK_MRST_A | CYCLONE_STATUS_MASK_MRST_B));
    TRY (cyclone_status_clear (module, CYCLONE_STATUS_MASK_MRST_A | CYCLONE_STATUS_MASK_MRST_B));
    
    /* wait for things to settle down */
    usleep(10);

    //TAB has 10 regular stratixes, GAB has 4.
    int gab;
    if (module == MOD_GAB){
	nreg   = 4;
	gab    = 1;
    } else {
	nreg   = 10;
	gab    = 0;
    }
        
    for (count = 0; count < nreg; count++) {
	if ((chipmask >> count) & mask) {	    
	    int chip = count; 
	    
	    TRY (stratix_pll_init(module, chip));	    
	    /* wait for things to settle down */
	    usleep(10);	    	    
	    TRY (stratix_status_read(module, chip, &value));
	    TRY (stratix_status_read(module, chip, &value));
	    if (gab) {
		if (value & LVDS_STATUS_PLL_LOCK_ERR) {
		    error("Stratix %d pll lock error", count);
		    return EXIT_FAILURE;
		}		
	    } else {	
		if (value & SWA_STATUS_PLL_LOCK_ERR) {
		    error("Stratix %d pll lock error", count);
		    return EXIT_FAILURE;
		}
	    }	    
	}
    }
    
    /* now reset the global chip */
    if ((chipmask >> nreg) & mask) {
	unsigned long chip = nreg; //In driver both TAB/GAB starts counting from 0.  
	
	TRY (stratix_pll_init(module, chip));
	/* wait for things to settle down */
	usleep(10);	
	
	if (gab) {
	    TRY (stratix_status_read(module, chip, &value));
	    TRY (stratix_status_read(module, chip, &value));
	    if (value & S30_STATUS_LOCKED == 0) {
		error("Stratix %d pll lock error", count);
		return EXIT_FAILURE;
	    }			    
	    //Try(Read_Gab_S30_Status(slot, &value));
	    //if (!(value & GAB30_PLL_LOCK)) {
	    //error("Stritix %d pll not locked", chip);
	    //return EXIT_FAILURE;
	    
	    //retVal = Read_Gab_S30_Status(slot, &value);
	    //if (!(value & GAB30_PLL_LOCK)) {
	    //error("Stritix %d pll not locked", chip);
	    //return EXIT_FAILURE;		       	
	} else {	
	    TRY (stratix_status_read(module, chip, &value));
	    TRY (stratix_status_read(module, chip, &value));
	    
	    if (!(value & GLOBAL_STATUS_CP90_LOCK)) {
		error("Global stratix's 90 MHz pll not locked");
		return EXIT_FAILURE;
	    }
	    if (!(value & GLOBAL_STATUS_CP53_LOCK)) {
		error("Global stratix's 53 MHz pll not locked");
		return EXIT_FAILURE;
	    }
	}
    }
	/* reset the scl state status again (i.e. whether in run or not) */
	return vmescl_generate_mrst();
}
 
/* Function that initialize stratix pll */
int stratix_pll_init(unsigned long module, unsigned long chip) 
{
    TRY(check_tab_or_gab(module));
    TRY(check_module_and_chip(module,chip));
    
    //only TAB global has different group for reset PLL...    
    if (chip == TAB_CHIP_GLOBAL) { /* note that GAB GLOBAL is not special, just TAB GLOBAL */
	return vmescl_write_serial(module, chip, GLOBAL_GROUP_RESET_PLL,0,0);
    } else {
	return vmescl_write_serial(module, chip, SWA_GROUP_RESET_PLL,0,0);
    }
}

//set the delay parameters
int stratix_set_scl_delay   (unsigned long module, unsigned long chip, unsigned long value){
    TRY(check_tab_or_gab(module));
    TRY(check_module_and_chip(module,chip));
    
    if (is_regular_stratix(module,chip)){    
	//GAB LVDS stratixes have same scl raw delay param as TAB swa,
	// at group SWA_GROUP_DELAY
	// SCL DELAY is delay with seradr "0"
	return vmescl_write_serial(module, chip, SWA_GROUP_DELAY, 0, value);
    } else {
	if (is_tab(module)){
	    //TAB global stratix has SCL delay at group GLOBAL_GROUP_WRITE_SCL_DELAY:
	    return vmescl_write_serial(module, chip, GLOBAL_GROUP_WRITE_SCL_DELAY, 0, value);	    
	}
	if (is_gab(module)){
	    return vmescl_write_serial(module, chip, S30_GROUP_PARAM_LOAD, S30_GROUP_PARAM_ADDR_SCL_DELAY, value);
	}	
    }   
    return EXIT_SUCCESS;
}

int stratix_set_raw_delay   (unsigned long module, unsigned long chip, unsigned long value){
    TRY(check_tab_or_gab(module));
    TRY(check_module_and_chip(module,chip));
    
    if (is_regular_stratix(module,chip)){
	if (is_gab(module)){
	    error("GAB LVDS chips do not have a raw delay!");
	    return EXIT_FAILURE;
	}	
	// For TAB SWA chips RAW DELAY is SWA_GROUP_DELAY with seradr "1"
	return vmescl_write_serial(module, chip, SWA_GROUP_DELAY, 1, value);	
    } else {
	if (is_tab(module)){
	    // For TAB global chips RAW DELAY is raw parameter with seradr "7":
	    vmescl_write_serial(module, TAB_CHIP_GLOBAL, GLOBAL_GROUP_WRITE_RAW_PAR, 7, value);
	}
	if (is_gab(module)){
	    return vmescl_write_serial(module, chip, S30_GROUP_PARAM_LOAD, S30_GROUP_PARAM_ADDR_L2DELAY, value);
	}	
    }   
    return EXIT_SUCCESS;
}

int stratix_set_accept_delay   (unsigned long module, unsigned long value){
    TRY(check_tab_or_gab(module));
    
    //for now it is just in GAB:
    TRY(check_gab(module));
    
    if (is_gab(module)){
	return vmescl_write_serial(module, GAB_CHIP_S30, S30_GROUP_PARAM_LOAD, S30_GROUP_PARAM_ADDR_ACCEPT_DELAY, value);
    }     
    return EXIT_FAILURE;

}

// set the input to REAL or TEST MEMORY
// TAB SWA modes:  ADF=0, TEST MEMORY=1
// GAB LVDS modes: TAB=0, TEST MEMORY=1

int stratix_set_input_mode(unsigned long module, unsigned long chip, int mode)
{
    TRY ( check_tab_or_gab(module) );
    TRY ( check_regular_stratix(module,chip));
        
    if (mode == 0) {
	//info("setting module %d chip %d to ADF mode.",module,chip);
	return stratix_status_write(module, chip, SWA_STATUS_MODE_ADF); 
    } else if (mode == 1) {	
	//info("setting module %d chip %d to TEST MEMORY mode.",module,chip);
	return stratix_status_write(module, chip, SWA_STATUS_MODE_TEST_MEMORY); 	
    }    
    return EXIT_SUCCESS;
}



