/*-------------------------------------------------------------------------------*/
  /*
    tcc_to_tabgab.h

    The L1Cal IIb TCC control code will interface to the TAB/GAB software 
    by sharing the source code developped to test and control the hardware.
    This collection of headers defines the routines that TCC uses to initialize, 
    control and monitor the TAB and GAB hardware.

    cf. http://www.pa.msu.edu/hep/d0/ftp/run2b/l1cal/tcc/coor/l1caltcc_to_tabgab_software_interface.txt

  */
#ifndef __tcc_to_tabgab__
#define __tcc_to_tabgab__
/*-------------------------------------------------------------------------------*/

/* ------------------------------------------------------------------------ */
/* ------------------------------------------------------------------------ */

/* --------------------------------------*/
/* 0.1.  Enumerated type for Return Value*/
/* --------------------------------------*/
/*
    This interface uses a dedicated enumerated data type for 
    all functions to use as return values.  This is common 
    to all routines described in this document.

    The TAB/GAB software should verify that the parameters passed are within
    the expected boundaries.   If an unexpected parameter is detected, 
    the function should return the enumerated value "EBadParam".
 
    The TAB/GAB software should verify that the requested action was 
    carried out successfully, e.g. by reading back all/some of the values 
    programmed in registers.  If an IO error is detected, the function
    should return the enumerated value "eIoError".

    When the TAB/GAB software has verify that the requested action 
    was successfully performed, the function should return 
    the enumerated value "eOk". 
*/

    enum EStatus
    {
        eOk       = 0,  /* Request filled, no problem*/
        eBadParam = 1,  /* Didn't like one of the parameters*/
	eBadEnv   = 2,  /* Problem with TAB/GAB Environment (not defined, non-existent directory, etc).*/
        eIoError  = 3   /* Tried to do as requested, but couldn't program hardware*/
    } ;

/* ------------------------------------*/
/* Constants for Parameter Boundaries  */
/* ------------------------------------*/

#define kiTabJetEtTreshMax     4095  // thresh limit for Tab_Set_JetEtThresh
#define kiTabEmEtTreshMax      4095  // thresh limit for Tab_Set_EmEtThresh
#define kiTabTtEtTreshMax       511  // thresh limit for Tab_Set_TT_Count_TT_Thresh

#define kiTabTauEtCoreCutMax   4095  // thresh limit for Tab_Set_Tau_EtCore
#define kiTabGlobSumEtCutMax    255  // thresh limit for Tab_Set_Global_Sum_Min_Et


/* ------------------------------------*/
/* TAB/GAB package version bookkeeping */ 
/* ------------------------------------*/

/*
    A TAB/GAB package version consists of an interoperable set of of
    firmware for TAB SWA, TAB global, GAB LVDS, and GAB S30 chips, and
    hardware configuration files.
   
    The package versions have cannonical form "version_X.Y".  A non-cannonical form,
    e.g. "testing" is assumed to be for special test purposes and the driver code
    will gracefully use the default test version number (1.1).
 
    With this routine, TCC communicates the package_version, and also the
    location of firmware and config file directories.
*/

    EStatus Tab_Gab_Set_Package_Version( const char * Package_Version,
			    	         const char * Firmware_Dir,
					 const char * Config_Dir,
					 const char * Work_Dir);
/*
    Return value:
        cf. EStatus definition

    Parameters:
        Package_Version
            see above
            const string
        Firmware_Dir
            firmware will be found in <Firmware_Dir>/<Package_Version>/
            const string
        Config_Dir
            hardware config files will be found in <Config_Dir>/<Package_Version>/
            const string
        Work_Dir
            any generated files for special tests will go in <Work_Dir>/
            const string
*/

/*-------------------------------------------------------------------------------*/
#endif // __tcc_to_tabgab__
