#include "tabgab/tcc_to_tabgab_coor.h"
#include "tabgab/tabgab_util.h"
#include "tabgab/vme.h"
#include "tabgab/vmescl.h"
#include "tabgab/tab.h"
#include "tabgab/gab.h"
#include "tabgab/kparams.h"
#include "tabgab/tabgab_util.h"
#include "tabgab/util.h"

//proto-types for locally used functions:
int usleep(unsigned int microseconds);

#define MAX_STRING 500

/* -----------------------------------------*/
/* VMESCL Hardware Level Initialization */
/* -----------------------------------------*/

EStatus Vmescl_Initialize () {
    SET_TRY_CONTEXT("Vmescl_Initialize");    
    CHECK_TABGAB_ENV;
    
    //Parameters:  I promise these will come from config file soon!
    unsigned long pulse_time     = 10;
    unsigned long pulse_width    = 32;
    unsigned long accept_time    = 47;
    unsigned long accept_freq    = 0;
    unsigned long scl_mask       = 0;
    int sleep_time               = 100000;

    //A master reset puts the entire system into a known state:
    SET_TRY_MSG(TRY_MSG, "failure generating mrst.");
    TRY_IO ( vmescl_generate_mrst() );

    //wait for master reset to finish:
    usleep(sleep_time);    
    
    //Set VMESCL parameters: 
    SET_TRY_MSG(TRY_MSG, "failure setting vmescl parameters.");
    TRY_IO ( vmescl_online() );
    TRY_IO ( vmescl_set_diag_pulse_time  (pulse_time   ) );
    TRY_IO ( vmescl_set_diag_pulse_width (pulse_width  ) );
    TRY_IO ( vmescl_set_accept_time      (accept_time  ) );
    TRY_IO ( vmescl_set_accept_freq      (accept_freq  ) );
    TRY_IO ( vmescl_set_scl_mask         (scl_mask     ) );

    return eOk;
}

/* -----------------------------------------*/
/* TAB Hardware Level Initialization */
/* -----------------------------------------*/
EStatus Tab_Initialize            ( unsigned long    Tab_Module ) {
    SET_TRY_CONTEXT("Tab_Initialize");    
    CHECK_TABGAB_ENV;

    SET_TRY_MSG(TRY_MSG, "failure checking parameters.");
    TRY_PAR(check_tab(Tab_Module));

    unsigned long module = Tab_Module;
    int chip ;
    
    //obtain hardware parameters from configuration file:
    unsigned swa_scl_delay = 0;
    unsigned swa_raw_delay = 0;
    unsigned glo_scl_delay = 0;
    unsigned glo_raw_delay = 0;
    unsigned swa_enable    = 0;

    char filename[MAX_STRING];
    kparams tab_params;

    snprintf(filename,MAX_STRING,"%s/%s/tab_param.txt",tabgab_config_dir,tabgab_package_version);
    info("Tab_Initial:  reading config file %s", filename);
    tab_params.parse_file (filename); 
    SET_TRY_MSG(TRY_MSG, "failure reading TAB parameters from config file.");
    TRY_ENV ( tab_params.get_unsigned ("swa_scl_delay",  swa_scl_delay ));
    TRY_ENV ( tab_params.get_unsigned ("swa_raw_delay",  swa_raw_delay ));
    TRY_ENV ( tab_params.get_unsigned ("glo_scl_delay",  glo_scl_delay ));
    TRY_ENV ( tab_params.get_unsigned ("glo_raw_delay",  glo_raw_delay ));
    TRY_ENV ( tab_params.get_unsigned ("swa_enable",     swa_enable    ));

    //input mode:  ADF=0, Test Memory = 1;
    int mode = 0;
        
    //reset all chips 
    SET_TRY_MSG(TRY_MSG, "failure resetting stratix chips.");
    TRY_IO ( stratix_reset               (module,0x7ff ));
        
    //clear the test memory to avoid potential confusion...
    SET_TRY_MSG(TRY_MSG, "failure clearing test memories.");
    TRY_IO ( swa_test_mem_clear          (module       ));
    
    SET_TRY_MSG(TRY_MSG, "failure setting SWA hardware parameters.");
    //chip level configuration:
    for (chip=0; chip<10; chip++){

	//enable deskew mode on channel-link LVDS receiver:
	TRY_IO( swa_set_enable (module, chip, swa_enable) );
	
	//set the input mode:
	TRY_IO( stratix_set_input_mode (module, chip, mode) );
	
	//set the SWA chip timing parameters:
	TRY_IO( stratix_set_scl_delay  (module, chip, swa_scl_delay) ); 
	TRY_IO( stratix_set_raw_delay  (module, chip, swa_raw_delay) );
    }

    //set the global chip parameters:
    SET_TRY_MSG(TRY_MSG, "failure setting global hardware parameters.");
    TRY_IO( stratix_set_scl_delay (module, TAB_CHIP_GLOBAL, glo_scl_delay) );
    TRY_IO( stratix_set_raw_delay (module, TAB_CHIP_GLOBAL, glo_raw_delay) );
        
    //set the global raw parameters (L2/L3 header):
    TRY_IO ( global_setup_level2_header(module) );
    
    //set parameters related to global energy calculation:
    TRY_IO ( global_setup_energy_params(module) );
    
    //send an init:
    SET_TRY_MSG(TRY_MSG, "failure generating init.");
    TRY_IO ( vmescl_generate_init() );
    
    
    //Make sure system comes up with clear status:
    // read all TAB chip status registers to clear transient errors during initialization:
    unsigned long status;
    SET_TRY_MSG(TRY_MSG, "failure clearing TAB status registers...");    
    for (chip=0; chip<=TAB_CHIP_GLOBAL; chip++){
	TRY_IO(stratix_status_read(module, chip, &status));
    }    
    return eOk;
}

/* -----------------------------------------*/
/* GAB Hardware Level Initialization        */
/* -----------------------------------------*/

EStatus Gab_Initialize            ( ) {    
    SET_TRY_CONTEXT("Tab_Initialize");    
    CHECK_TABGAB_ENV;
    
    unsigned long module = MOD_GAB;
    
    //obtain hardware parameters from configuration file:
    unsigned lvds_scl_delay    = 0;
    unsigned s30_scl_delay     = 0;
    unsigned s30_raw_delay     = 0;
    unsigned s30_accept_delay  = 0;    
    unsigned special_bc        = 0;
    int chip;
    
    char filename[MAX_STRING];
    kparams params;

    snprintf(filename,MAX_STRING,"%s/%s/gab_param.txt",tabgab_config_dir,tabgab_package_version);
    params.parse_file (filename); 
    SET_TRY_MSG(TRY_MSG, "failure reading GAB parameters from file.");
    TRY_ENV ( params.get_unsigned ("lvds_scl_delay",   lvds_scl_delay   ));
    TRY_ENV ( params.get_unsigned ("s30_scl_delay",    s30_scl_delay    ));
    TRY_ENV ( params.get_unsigned ("s30_raw_delay",    s30_raw_delay    ));
    TRY_ENV ( params.get_unsigned ("s30_accept_delay", s30_accept_delay )); 
    TRY_ENV ( params.get_unsigned ("special_bc",       special_bc       ));
        
    //input mode:  TAB=0, Test Memory = 1;
    int mode = 0;    

    SET_TRY_MSG(TRY_MSG, "failure setting up GAB LVDS chip parameters...");    
    //chip level configuration:
    for (chip=0; chip<4; chip++){
	//set the LVDS chip scl delay parameters:

	TRY_IO ( stratix_set_scl_delay (module, chip, lvds_scl_delay) );
	//set the input mode:

	TRY_IO ( stratix_set_input_mode    (module, chip, mode)       );
    }
    
    SET_TRY_MSG(TRY_MSG, "failure setting up GAB S30 parameters...");    
    
    //set the S30 chip timing parameters:
    TRY_IO( stratix_set_scl_delay    (module, GAB_CHIP_S30, s30_scl_delay) );
    TRY_IO( stratix_set_raw_delay    (module, GAB_CHIP_S30, s30_raw_delay) );    
    TRY_IO( stratix_set_accept_delay (module, s30_accept_delay)            );
    
    //set special trigger to BC10:
    TRY_IO( s30_load_parameter(S30_GROUP_PARAM_ADDR_SPECIALNUM, special_bc) );
    
    SET_TRY_MSG(TRY_MSG, "failure setting up GAB S30 L2/L3 header...");    
    //set the L2/L3 buffer parameters:
    TRY_IO (s30_setup_level2_header() );

    SET_TRY_MSG(TRY_MSG, "failure clearing GAB LVDS test memories...");    
    //clear test memory to avoid confusion!    
    TRY_IO( lvds_test_mem_clear() );
            
    //Make sure system comes up with clear status:
    // read lvds chip status to clear transient errors during initialization:
    unsigned long status;
    SET_TRY_MSG(TRY_MSG, "failure clearing GAB status registers...");    
    for (chip=0; chip<=GAB_CHIP_S30; chip++){
	TRY_IO(stratix_status_read     (module, chip, &status));
    }

    //send an init to clear S30 status now that everyone is happy!
    vmescl_generate_init();
    
    return eOk;
}

/* ---------------------------*/
/* Jet Reference Sets         */
/* ---------------------------*/

EStatus Tab_Set_JetEtThresh       ( unsigned long    Tab_Module,
				    unsigned long    Tab_Chip,
				    unsigned long    Tab_Thresh_Num,
				    unsigned long    Set_AdcCount_Thresh ) {
    SET_TRY_CONTEXT("Tab_Set_JetEtThresh");
    SET_TRY_MSG(TRY_MSG, "failure setting Jet Et Thresh");
    
    TRY_PAR(check_tab(Tab_Module));
    TRY_PAR(check_regular_stratix(Tab_Module, Tab_Chip));
    if ((Tab_Thresh_Num < 0) || (Tab_Thresh_Num > 6)){ 
	error("Tab_Set_TauEtThresh:  Tab_Thresh_Num %d out of allowed range!", Tab_Thresh_Num);
	return eBadParam;
    }
    
    TRY_IO( swa_load_param_jet(Tab_Module, Tab_Chip, Tab_Thresh_Num, Set_AdcCount_Thresh) );
    
    return eOk;
}


/* ---------------------------*/
/* EM  Reference Sets         */
/* ---------------------------*/

EStatus Tab_Set_EmEtThresh        ( unsigned long    Tab_Module,
				    unsigned long    Tab_Chip,
				    unsigned long    Tab_Thresh_Num,
				    unsigned long    Set_AdcCount_Thresh ){
    SET_TRY_CONTEXT("Tab_Set_EmEtThresh");
    SET_TRY_MSG(TRY_MSG, "failure setting EM Et Thresh");
    
    TRY_PAR(check_tab(Tab_Module));
    TRY_PAR(check_regular_stratix(Tab_Module, Tab_Chip));
    if ((Tab_Thresh_Num < 0) || (Tab_Thresh_Num > 6)){ 
	error("Tab_Set_TauEtThresh:  Tab_Thresh_Num %d out of allowed range!", Tab_Thresh_Num);
	return eBadParam;
    }
        
    TRY_IO( swa_load_param_em(Tab_Module, Tab_Chip, Tab_Thresh_Num, Set_AdcCount_Thresh));
    
    return eOk;
}

/* -------------------------------------*/
/* Set ICR enables                      */
/* -------------------------------------*/

EStatus Tab_Set_ICR_Enables       ( unsigned long    Tab_Module,
				    unsigned long    Tab_Chip,
				    unsigned long    Set_Jet_Enable,
				    unsigned long    Set_Global_Sums_Enable){
    SET_TRY_CONTEXT("Tab_Set_ICR_Enables");
    SET_TRY_MSG(TRY_MSG, "failure setting HW parameters");
    
    TRY_PAR(check_tab(Tab_Module));
    TRY_PAR(check_regular_stratix(Tab_Module, Tab_Chip));

    if ((Set_Jet_Enable > 1) || (Set_Global_Sums_Enable > 1)){ 
	error("Set_Global_Sums_Enable:  Enable parameters must be 0 or 1");
	return eBadParam;
    }        
    unsigned enable = 0;        
    enable |= (Set_Jet_Enable         << 0);
    enable |= (Set_Global_Sums_Enable << 1);
    	
    TRY_IO(swa_set_enable(Tab_Module, Tab_Chip, enable));
        
    return eOk;
}

/* -------------------------------------*/
/* Set EM Isolation Parameters          */
/* -------------------------------------*/

unsigned Hack_Previous_Write_Of_Em_Param_9 = 0x0;

EStatus Tab_Set_Em_Isolation      ( unsigned long    Tab_Module,
                                    unsigned long    Tab_Chip,
                                    unsigned long    Set_EmIsol_Enable,
				    unsigned long    Set_EmHd_Enable,
				    unsigned long    Set_EmIsol_Ref_Set){
    SET_TRY_CONTEXT("Tab_Set_Em_Isolation");
    SET_TRY_MSG(TRY_MSG, "failure setting hardware parameters");
    
    TRY_PAR(check_tab(Tab_Module));
    TRY_PAR(check_regular_stratix(Tab_Module, Tab_Chip));
    
    if ((Set_EmIsol_Enable > 1) || (Set_EmHd_Enable > 1)){ 
	error("Tab_Set_Em_Isolation:  Enable parameters must be 0 or 1");
	return eBadParam;
    }        
    if ((Set_EmIsol_Ref_Set > 7)){ 
	error("Tab_Set_Em_Isolation:  Min Ref Set must be in 0:7");
	return eBadParam;
    }        
    
    unsigned vme_em_param_7 = 0x0; 
    unsigned vme_em_param_8 = 0x0; 
    unsigned vme_em_param_9 = 0x0; 
    
    //em enables:
    vme_em_param_8 |= (2 << 0);  //isolation param "a" now hardwired for minimal latency
    vme_em_param_8 |= (3 << 2);  //isolation param "b" now hardwired for minimal latency    
    vme_em_param_8 |= (Set_EmIsol_Enable << 4);
    vme_em_param_8 |= (Set_EmHd_Enable << 5);
    vme_em_param_8 |= (0 << 6);  //disable neighbor masking in EM algorithm.    
        
    //hack for shared/non-readable (ugh!) VME parameter:    
    // -> keep any previously written bits:
    vme_em_param_9 |= (Hack_Previous_Write_Of_Em_Param_9 & 0x7);
    vme_em_param_9 |= (Set_EmIsol_Ref_Set << 3);
    Hack_Previous_Write_Of_Em_Param_9 = vme_em_param_9;
            
    TRY_IO( swa_load_param_em  ( Tab_Module, Tab_Chip, 7, vme_em_param_7));
    TRY_IO( swa_load_param_em  ( Tab_Module, Tab_Chip, 8, vme_em_param_8));
    TRY_IO( swa_load_param_em  ( Tab_Module, Tab_Chip, 9, vme_em_param_9));
    
    return eOk;
}

/* -------------------------------------*/
/* Set L2 Mask Reference Sets           */
/* -------------------------------------*/

EStatus Tab_Set_Mask_Ref_Sets     ( unsigned long    Tab_Module,
                                    unsigned long    Tab_Chip,
				    unsigned long    Set_Em_Mask_Ref_Set,
                                    unsigned long    Set_Jet_Mask_Ref_Set,
				    unsigned long    Set_Tau_Mask_Ref_Set ){
    SET_TRY_CONTEXT("Tab_Set_Mask_Ref_Sets");
    SET_TRY_MSG(TRY_MSG, "failure setting hardware parameters");
    
    TRY_PAR(check_tab(Tab_Module));
    TRY_PAR(check_regular_stratix(Tab_Module, Tab_Chip));

    if ((Set_Em_Mask_Ref_Set > 7) || (Set_Jet_Mask_Ref_Set > 7) || (Set_Tau_Mask_Ref_Set > 7)){	
	error("Tab_Set_Mask_Ref_Sets:  Min Ref Set must be in 0:7");
	return eBadParam;
    }        
    
    unsigned vme_em_param_9  = 0x0; 
    unsigned vme_jet_param_7 = 0x0; 
    
    //hack for shared/non-readable (ugh!) VME parameter:    
    // -> keep any previously written bits:
    vme_em_param_9 |= (Hack_Previous_Write_Of_Em_Param_9 & 0x38);
    vme_em_param_9 |= Set_Em_Mask_Ref_Set;
    Hack_Previous_Write_Of_Em_Param_9 = vme_em_param_9;
            
    vme_jet_param_7 |= (Set_Tau_Mask_Ref_Set << 3);
    vme_jet_param_7 |= (Set_Jet_Mask_Ref_Set << 0);
        
    TRY_IO( swa_load_param_em  ( Tab_Module, Tab_Chip, 9, vme_em_param_9  ));
    TRY_IO( swa_load_param_jet ( Tab_Module, Tab_Chip, 7, vme_jet_param_7 ));
    
    return eOk;
}


/* ------------------------------------*/
/* Set Tau Minimum Et Core             */
/* ------------------------------------*/

EStatus Tab_Set_TauEtCoreCut        ( unsigned long    Tab_Module,
			   	      unsigned long    Tab_Chip,
				      unsigned long    Set_TauEtCoreCut ){
    SET_TRY_CONTEXT("Tab_Set_TauEtCoreCut");
    SET_TRY_MSG(TRY_MSG, "failure setting hardware parameters");
    
    TRY_PAR( check_tab(Tab_Module));
    TRY_PAR( check_regular_stratix(Tab_Module, Tab_Chip));
    
    if (Set_TauEtCoreCut > 4095){	
	error("Tab_Set_TauEtCoreCut:  TauEtCoreCut must be in 0:4095");
	return eBadParam;
    }        
    
    TRY_IO( swa_load_param_jet ( Tab_Module, Tab_Chip, 8, Set_TauEtCoreCut));
	    
    return eOk;
}

/* ------------------------------------*/
/* Set Global Sum Minimum Et           */
/* ------------------------------------*/

EStatus Tab_Set_GlobalSumEtCut ( unsigned long    Tab_Module,
				 unsigned long    Tab_Chip,
				 unsigned long    Set_AdcCount_Thresh ){
    SET_TRY_CONTEXT("Tab_Set_GlobalSumEtCut");
    SET_TRY_MSG(TRY_MSG, "failure setting hardware parameters");
    
    TRY_PAR(check_tab(Tab_Module));
    TRY_PAR(check_regular_stratix(Tab_Module, Tab_Chip));
    
    if (Set_AdcCount_Thresh > 255){	
	error("Tab_Set_Global_Sum_Min_Et:  Sum_AdcCount_Thresh must be in 0:255");
	return eBadParam;
    }        
    
    TRY_IO(swa_set_minimum_energy (Tab_Module, Tab_Chip, Set_AdcCount_Thresh));
	
    return eOk;
}

/* ------------------------------------*/
/* Set Eta 38                          */
/* ------------------------------------*/

EStatus Tab_Set_Eta38 ( unsigned long    Tab_Module,
			unsigned long    Tab_Chip,
			unsigned long    Set_Eta38 ){
    SET_TRY_CONTEXT("Tab_Set_Eta38");
    SET_TRY_MSG(TRY_MSG, "failure setting hardware parameters");
    
    TRY_PAR(check_tab(Tab_Module));
    TRY_PAR(check_regular_stratix(Tab_Module, Tab_Chip));

    if (Set_Eta38 > 255){	
	error("Tab_Set_Eta_38:  Eta38 must be in 0:255");
	return eBadParam;
    }        
    
    TRY_IO(swa_set_eta_38 (Tab_Module, Tab_Chip, Set_Eta38));
        
    return eOk;
}

/* --------------------------------------------------------------------------*/
/* Set TT Thresh for TT Count (aka "noise trigger", "run 2a trigger")        */
/* --------------------------------------------------------------------------*/

EStatus Tab_Set_TT_Count_TT_Thresh ( unsigned long    Tab_Module,
				     unsigned long    Tab_Chip,
				     unsigned long    Thresh_Num,
				     unsigned long    Set_TT_Thresh){    
    SET_TRY_CONTEXT("Tab_Set_TT_Count_TT_Thresh");
    SET_TRY_MSG(TRY_MSG, "failure setting hardware parameters");
    
    TRY_PAR(check_tab(Tab_Module));
    TRY_PAR(check_regular_stratix(Tab_Module, Tab_Chip));

    if (Thresh_Num > 2){
	error("Tab_Set_TT_Count_TT_Thresh:  Thresh_Num must be in 0:2");
	return eBadParam;
    }
    
    if (Set_TT_Thresh > 511){	
	error("Tab_Set_TT_Count_TT_Thresh:  TT_Thresh must be in 0:511");
	return eBadParam;
    }        
    
    unsigned seradr[3] = {SWA_GROUP_DELAY_SERADR_NOISE1, SWA_GROUP_DELAY_SERADR_NOISE2, SWA_GROUP_DELAY_SERADR_NOISE3}; 
    
    TRY_IO( vmescl_write_serial(Tab_Module, Tab_Chip, SWA_GROUP_DELAY, seradr[Thresh_Num], Set_TT_Thresh) );

    return eOk;
}

/* --------------------------------------------------------------------------*/
/* Set Count Thresh for TT Count (aka "noise trigger", "run 2a trigger")        */
/* --------------------------------------------------------------------------*/

EStatus Gab_Set_TT_Count_Thresh ( unsigned long    Thresh_Num,
				  unsigned long    Set_Count_Thresh){
    SET_TRY_CONTEXT("Gab_Set_TT_Count_Thresh");
    SET_TRY_MSG(TRY_MSG, "failure setting hardware parameters");
    
    if (Thresh_Num > 2){
	error("Gab_Set_TT_Count_Thresh:  Thresh_Num must be in 0:2");
	return eBadParam;
    }
    
    if (Set_Count_Thresh > 2047){
	error("Gab_Set_TT_Count_Thresh:  TT_Thresh must be in 0:2047");
	return eBadParam;
    }        
    
    unsigned seradr[3] = {S30_GROUP_PARAM_ADDR_NOISE0, S30_GROUP_PARAM_ADDR_NOISE1, S30_GROUP_PARAM_ADDR_NOISE2};

    TRY_IO (s30_load_parameter(seradr[Thresh_Num], Set_Count_Thresh));
    
    return eOk;
}

/* ------------------------------------*/
/* IV.1.a. Jet Count over full coverage*/
/* ------------------------------------*/

EStatus Gab_Set_JetAllTerm        ( unsigned long    Rel_Term_Num,
				    unsigned long    Use_Thresh_Num,
				    unsigned long    Set_ObjCount_Thresh ) {    
    return eOk;
}

/* ---------------------------------------------------*/
/* IV.1.b. Jet Count over Central Calorimeter coverage*/
/* ---------------------------------------------------*/

EStatus Gab_Set_JetCentralTerm    ( unsigned long    Rel_Term_Num,
				    unsigned long    Use_Thresh_Num,
				    unsigned long    Set_ObjCount_Thresh ){    
    return eOk;
}

/* -----------------------------------*/
/* IV.2.a. EM Count over full coverage*/
/* -----------------------------------*/

EStatus Gab_Set_EmAllTerm         ( unsigned long    Rel_Term_Num,
				    unsigned long    Use_Thresh_Num,
				    unsigned long    Set_ObjCount_Thresh ) {
    return eOk;
}

/* --------------------------------------------------*/
/* IV.2.b. EM Count over Central Calorimeter coverage*/
/* --------------------------------------------------*/

EStatus Gab_Set_EmCentralTerm     ( unsigned long    Rel_Term_Num,
				    unsigned long    Use_Thresh_Num,
				    unsigned long    Set_ObjCount_Thresh ){
    return eOk;
}


/* ------------------------------------*/
/* IV.3.a. Tau Count over full coverage*/
/* ------------------------------------*/

EStatus Gab_Set_TauAllTerm        ( unsigned long    Rel_Term_Num,
				    unsigned long    Use_Thresh_Num,
				    unsigned long    Use_Ratio_Num,
				    unsigned long    Set_ObjCount_Thresh ){
    return eOk;
}

EStatus Gab_Set_TauCentralTerm        ( unsigned long    Rel_Term_Num,
				    unsigned long    Use_Thresh_Num,
				    unsigned long    Use_Ratio_Num,
				    unsigned long    Set_ObjCount_Thresh ){
    return eOk;
}
/* ------------------------------------------------------------------------ */
/* ------------------------------------------------------------------------ */

/* --------------------------------------------*/
/* IV.4.a. EM Isolated Count over full coverage*/
/* --------------------------------------------*/

EStatus Gab_Set_IsolEmAllTerm     ( unsigned long    Rel_Term_Num,
				    unsigned long    Use_Thresh_Num,
				    unsigned long    Set_ObjCount_Thresh ){    
    return eOk;
}



/* -----------------------------------------------------------*/
/* IV.4.b. EM Isolated Count over Central Calorimeter coverage*/
/* -----------------------------------------------------------*/

EStatus Gab_Set_IsolEmCentralTerm ( unsigned long    Rel_Term_Num,
				    unsigned long    Use_Thresh_Num,
				    unsigned long    Set_ObjCount_Thresh ){    
    return eOk;
}

/* ---------------------------*/
/* IV.5. Missing Et Energy Cut*/
/* ---------------------------*/

EStatus Gab_Set_MissEtTerm        ( unsigned long    Rel_Term_Num,
				    unsigned long    Set_AdcSquare_Thresh ){
    
    return eOk;
}

/* -------------------------*/
/* IV.6. Total Et Energy Cut*/
/* -------------------------*/

EStatus Gab_Set_TotEtTerm         ( unsigned long    Rel_Term_Num,
				    unsigned long    Set_AdcCount_Thresh ){
    return eOk;
}

/* -------------------------------------------*/
/* IV.7. Topological Acoplanar Jet Requirement*/
/* -------------------------------------------*/

EStatus Gab_Set_AcoplanarJetTerm  ( unsigned long    Rel_Term_Num,
				    unsigned long    Use_Thresh_Num1,
				    unsigned long    Use_Thresh_Num2,
				    unsigned long    DeltaPhi ){
    return eOk;
}

/* -----------------------------------*/
/* IV.8. Non collinear low Et Jet term*/
/* -----------------------------------*/

EStatus Gab_Set_NonColLowEtJetTerm( unsigned long    Rel_Term_Num,
				    unsigned long    Use_Thresh_Num1,
				    unsigned long    Use_Thresh_Num2,
				    unsigned long    DeltaPhi ){
    return eOk;
}

/* ---------------------------------*/
/* IV.9. Non collinear Mono Jet term*/
/* ---------------------------------*/

EStatus Gab_Set_NonColMonoJetTerm ( unsigned long    Rel_Term_Num,
				    unsigned long    Use_Thresh_Num1,
				    unsigned long    Use_Thresh_Num2,
				    unsigned long    DeltaPhi ){
    return eOk;
}

/* ----------------------------------------------*/
/* IV.10. Topological Back-to-Back EM Requirement*/
/* ----------------------------------------------*/

EStatus Gab_Set_BackToBackEmTerm  ( unsigned long    Rel_Term_Num,
				    unsigned long    Use_Thresh_Num,
				    unsigned long    DeltaPhi ){
    return eOk;
}

/* ----------------------------------*/
/* IV.11. Topological Jet Free Region*/
/* ----------------------------------*/

EStatus Gab_Set_JetFreeTerm       ( unsigned long    Rel_Term_Num,
				    unsigned long    Use_Thresh_Num_a,
				    unsigned long    Use_Thresh_Num_v,
				    unsigned long    DeltaPhi ){
    return eOk;
}

