/*-------------------------------------------------------------------------------*/
  /*
    tcc_to_tabgab_monit.h

    This header defines the routines that TCC uses
    to retrieve monitoring information from the TAB and GAB hardware.

  */

#ifndef __tcc_to_tabgab_monit__
#define __tcc_to_tabgab_monit__
/*-------------------------------------------------------------------------------*/

#include "tabgab/tcc_to_tabgab.h"

/* ------------------------------------------------------------------------ */
/* ------------------------------------------------------------------------ */


/* -----------------------------------------*/
/* I.   TAB Algorithm Chip Status           */
/* -----------------------------------------*/

    EStatus Tab_Monit_AlgoStatus      ( unsigned long    Tab_Module,
                                        unsigned long    Tab_Chip,
                                        unsigned short*  pMonit_Status ) ;

/*
    Return value:
        cf. EStatus definition

        The monitoring information iself is returned through a parameter, cf. below.

    Parameters:
        Tab_Module
            Module ID of TAB card
            unsigned integer
            allowed values = 0:7
        Tab_Chip
            Chip ID on TAB Card
            unsigned integer
            allowed values = 0:9
        pMonit_Status
            pointer to unsigned integer
            The calling routine is responsible for providing a valid pointer
            This routine will write one 16 bit word at the address specified

    Usage Note:

        TCC will call this routine to fill the following monitoring variable
        cf. trigmon/L1CalTcc_MonitData_TabGab_Status.hpp

    // These are the Algorithm TAB FPGA Status words,
    // with the second index of the array (0-9) corresponds to TAB "Chips 0-9"
    //
    // Cable A:  (nominal eta, nominal phi + 4) 
    // Cable B:  (nominal eta, nominal phi    )
    // Cable C:  (nominal eta, nominal phi - 4)
    //
    //   Bit Number     Bit Meaning
    //
    //       0          parity error on input cables C
    //       1          parity error on input cables B
    //       2          parity error on input cables A
    //       3          data sync error on input cables C
    //       4          data sync error on input cables B
    //       5          data sync error on input cables A
    //       6          BC mismatch on input cables C
    //       7          BC mismatch on input cables B
    //       8          BC mismatch on input cables A
    //       9          PLL lost lock
    //       10         mode_ff
    //       11         PRN test data error on cables C
    //       12         PRN test data error on cables B
    //       13         PRN test data error on cables A
    //       14         Expert Bit 1 (changes for spec tests)
    //       15         Expert Bit 2 (changes for spec tests)
    //

    uint16                          a2uwTabAlgo_Status  [kL1CalTcc_Tot_Tab_Card]
                                                        [kL1CalTcc_Tot_Tab_Algo_Fpga] ;

*/


/* -----------------------------------------*/
/* II.   TAB Global Chip Status             */
/* -----------------------------------------*/

    EStatus Tab_Monit_GlobalStatus    ( unsigned long    Tab_Module,
                                        unsigned short*  pMonit_Status ) ;

/*
    Return value:
        cf. EStatus definition

        The monitoring information iself is returned through a parameter, cf. below.

    Parameters:
        Tab_Module
            Module ID of TAB card
            unsigned integer
            allowed values = 0:7
        pMonit_Status
            pointer to unsigned integer
            The calling routine is responsible for providing a valid pointer
            This routine will write one 16 bit word at the address specified

    Usage Note:

        TCC will call this routine to fill the following monitoring variable
        cf. trigmon/L1CalTcc_MonitData_TabGab_Status.hpp

    // These are the Global FPGA Status words from the TAB Global FPGA, i.e. TAB "Chip 10"
    //
    //   Bit Number     Bit Meaning
    //
    //       0          testmode data
    //       1          cp90 locked
    //       2          cp53 locked
    //       3          run           (saw init and turn)
    //      4:15        Unused
    //

    uint16                          auwTabGlob_Status   [kL1CalTcc_Tot_Tab_Card] ;

*/


/* -----------------------------------------*/
/* III.   GAB Reveiver Chip Status          */
/* -----------------------------------------*/


    EStatus Gab_Monit_ReceiverStatus  ( unsigned long    Gab_Chip,
                                        unsigned short*  pMonit_Status ) ;

/*
    Return value:
        cf. EStatus definition

        The monitoring information iself is returned through a parameter, cf. below.

    Parameters:
        Gab_Chip
            Chip ID on GAB Card
            unsigned integer
            allowed values = 0:3
        pMonit_Status
            pointer to unsigned integer
            The calling routine is responsible for providing a valid pointer
            This routine will write one 16 bit word at the address specified

    Usage Note:

        TCC will call this routine to fill the following monitoring variable
        cf. trigmon/L1CalTcc_MonitData_TabGab_Status.hpp

    // These are the Receiver GAB FPGA Status words,
    // with the array index (0-3) corresponding to the GAB "LVDS Chips 1-4"
    // Note that each LVDS chip in the GAB has inputs from two TABs, hence
    // the cable 1 / cable 2 below.
    //
    //   Bit Number     Bit Meaning
    //
    //       0          pulse:  diagnostic pulse signal is present
    //       1          PLL not locked
    //       2          latched parity error on input cable 1
    //       3          latched parity error on input cable 2
    //       4          unlatched parity error on input cable 1
    //       5          unlatched parity error on input cable 2
    //       6          sync_error:    if sync signal on two cables differs
    //       7          bc_error:      if bc signal on two cables differs
    //       8          valid_error:   if valid signal on two cables differs
    //       9          sclbc_error:   if chip10 BC on two cables differs
    //       10         sclturn_error: if turn on two cables differs
    //       11-15      Unused
  
    uint16                          auwGabRec_Status    [kL1CalTcc_Tot_Gab_Rec_Fpga] ;

*/


/* -----------------------------------------*/
/* IV.   GAB Global Chip Status             */
/* -----------------------------------------*/


    EStatus Gab_Monit_GlobalStatus    ( unsigned short*  pMonit_Status ) ;

/*
    Return value:
        cf. EStatus definition

        The monitoring information iself is returned through a parameter, cf. below.

    Parameters:
        pMonit_Status
            pointer to unsigned integer
            The calling routine is responsible for providing a valid pointer
            This routine will write one 16 bit word at the address specified

    Usage Note:

        TCC will call this routine to fill the following monitoring variable
        cf. trigmon/L1CalTcc_MonitData_TabGab_Status.hpp

    // This is the Global GAB FPGA Status words, i.e. GAB "S-30 FPGA"
    //
    //   Bit Number     Bit Meaning
    //
    //       0          init        init signal is present
    //       1          sclrun:     saw init and turn
    //       2          locked:     PLLs locked
    //       3          sclerror:   SCL Init requested
    //      4:15        Unused
    //

    uint16                          uwGabGlob_Status ;

*/

#endif // __tcc_to_tabgab_monit__
